(function ($, Drupal, drupalSettings, once) {
  'use strict';

  /**
   * ENHANCED CKEditor5 support for AJAX-loaded Component Field with proper re-initialization
   */
  Drupal.behaviors.componentFieldCKEditorEnhanced = {
    attach: function (context, settings) {

      // Handle text_format fields in Component Field with enhanced detection
      $(once('component-field-ckeditor-enhanced', '.component-field-config-field[data-text-format], .text-format-wrapper', context)).each(function () {
        initializeTextFormatEnhanced($(this));
      });

      // Enhanced AJAX completion handler specifically for CKEditor5
      $(document).off('ajaxComplete.componentFieldCKEditorEnhanced')
        .on('ajaxComplete.componentFieldCKEditorEnhanced', function (event, xhr, settings) {
          // Check if this was a Component Field AJAX request
          if (settings.url && (settings.url.indexOf('component-field') !== -1 || settings.url.indexOf('system/ajax') !== -1)) {
            // Delay to ensure DOM is fully updated
            setTimeout(function () {
              reInitializeCKEditorEnhanced(context);
            }, 200);
          }
        });

      // Listen for custom CKEditor reinitialization trigger
      $(document).off('ckeditorReinitialized.componentField')
        .on('ckeditorReinitialized.componentField', function () {
          setTimeout(function () {
            reInitializeCKEditorEnhanced(document);
          }, 300);
        });

      function initializeTextFormatEnhanced($element) {
        try {
          var $textFormat = $element.hasClass('text-format-wrapper') ? $element : $element.find('.text-format-wrapper');

          if ($textFormat.length) {
            // Ensure unique IDs to prevent conflicts
            var $textarea = $textFormat.find('textarea');
            if ($textarea.length && !$textarea.attr('id')) {
              var uniqueId = 'textarea-' + Math.random().toString(36).substr(2, 9);
              $textarea.attr('id', uniqueId);
            }

            // Mark as processed to prevent duplicate initialization
            $textFormat.addClass('component-field-ckeditor-processed');

            // Ensure CKEditor5 behaviors are attached with error handling
            try {
              if (typeof Drupal.behaviors.editor !== 'undefined') {
                Drupal.behaviors.editor.attach($textFormat[0], drupalSettings);
              }

              if (typeof Drupal.behaviors.ckeditor5 !== 'undefined') {
                Drupal.behaviors.ckeditor5.attach($textFormat[0], drupalSettings);
              }
            } catch (error) {
              console.warn('CKEditor5 initialization warning:', error);
              // Continue without breaking the form
            }
          }
        } catch (error) {
          console.error('Error initializing text format field:', error);
        }
      }

      function reInitializeCKEditorEnhanced(context) {
        try {
          // Find all text-format wrappers that need re-initialization
          $('.text-format-wrapper:not(.component-field-ckeditor-processed)', context).each(function () {
            var $wrapper = $(this);
            var $textarea = $wrapper.find('textarea');

            if ($textarea.length) {
              var textareaId = $textarea.attr('id');

              // Handle CKEditor5 instances
              if (textareaId) {
                // Destroy existing CKEditor5 instance if it exists
                if (typeof Drupal.CKEditor5Instances !== 'undefined' && Drupal.CKEditor5Instances[textareaId]) {
                  try {
                    Drupal.CKEditor5Instances[textareaId].destroy().catch(function (error) {
                      console.warn('CKEditor5 destroy warning:', error);
                    });
                    delete Drupal.CKEditor5Instances[textareaId];
                  } catch (error) {
                    console.warn('Error destroying CKEditor5 instance:', error);
                  }
                }

                // Handle legacy CKEditor instances
                if (typeof CKEDITOR !== 'undefined' && CKEDITOR.instances[textareaId]) {
                  try {
                    CKEDITOR.instances[textareaId].destroy();
                  } catch (error) {
                    console.warn('Error destroying CKEditor instance:', error);
                  }
                }
              }
            }

            // Re-attach all relevant Drupal behaviors
            try {
              // Core editor behavior
              if (typeof Drupal.behaviors.editor !== 'undefined') {
                Drupal.behaviors.editor.attach($wrapper[0], drupalSettings);
              }

              // CKEditor5 behavior
              if (typeof Drupal.behaviors.ckeditor5 !== 'undefined') {
                Drupal.behaviors.ckeditor5.attach($wrapper[0], drupalSettings);
              }

              // Filter behavior for format selection
              if (typeof Drupal.behaviors.filterStatus !== 'undefined') {
                Drupal.behaviors.filterStatus.attach($wrapper[0], drupalSettings);
              }

              // Text editor behavior
              if (typeof Drupal.behaviors.textEditor !== 'undefined') {
                Drupal.behaviors.textEditor.attach($wrapper[0], drupalSettings);
              }

              // Mark as processed
              $wrapper.addClass('component-field-ckeditor-processed');

            } catch (error) {
              console.error('Error re-attaching editor behaviors:', error);
            }
          });

          // Also handle any unprocessed text format fields
          $('.component-field-config-field[data-text-format]:not(.component-field-ckeditor-processed)', context).each(function () {
            initializeTextFormatEnhanced($(this));
          });

        } catch (error) {
          console.error('Error in reInitializeCKEditorEnhanced:', error);
        }
      }
    }
  };

  /**
   * Enhanced widget behavior with IMPROVED CKEditor5 support
   */
  Drupal.behaviors.componentFieldWidgetCKEditor = {
    attach: function (context, settings) {

      // Initialize component items with enhanced CKEditor handling
      $(once('component-field-widget-ckeditor', '.component-field-item', context)).each(function () {
        var $item = $(this);

        // Handle text_format field changes for preview updates with debouncing
        var previewUpdateTimeout;

        $item.find('.text-format-wrapper textarea, .text-format-wrapper select').on('input change keyup', function () {
          clearTimeout(previewUpdateTimeout);
          previewUpdateTimeout = setTimeout(function () {
            updateComponentPreviewWithTextFormat($item);
          }, 500); // Debounce to prevent excessive updates
        });

        // Handle CKEditor5 content changes
        $item.on('CKEditor5:change', function () {
          clearTimeout(previewUpdateTimeout);
          previewUpdateTimeout = setTimeout(function () {
            updateComponentPreviewWithTextFormat($item);
          }, 500);
        });
      });

      function updateComponentPreviewWithTextFormat($item) {
        try {
          var $preview = $item.find('.component-field-preview-content');
          if (!$preview.length) {
            return;
          }

          var componentType = $item.find('select[name*="component_type"]').val();
          if (!componentType) {
            return;
          }

          // Collect configuration including text_format fields
          var config = {};

          $item.find('.component-field-config-field').each(function () {
            var $field = $(this);
            var propName = $field.data('prop-name') || extractPropName($field.attr('name'));

            if (propName) {
              var value = getFieldValueEnhanced($field);
              if (value !== NULL && value !== '') {
                config[propName] = value;
              }
            }
          });

          // Generate and display preview
          generatePreviewEnhanced(componentType, config, function (html) {
            $preview.html(html);
          });

        } catch (error) {
          console.error('Error updating preview with text format:', error);
        }
      }

      function getFieldValueEnhanced($field) {
        try {
          // Handle text_format fields with CKEditor5 content
          if ($field.hasClass('text-format-wrapper') || $field.find('.text-format-wrapper').length) {
            var $wrapper = $field.hasClass('text-format-wrapper') ? $field : $field.find('.text-format-wrapper');
            var $textarea = $wrapper.find('textarea');
            var $formatSelect = $wrapper.find('select[name*="[format]"]');

            if ($textarea.length) {
              var textareaId = $textarea.attr('id');
              var value = $textarea.val();

              // Try to get content from CKEditor5 instance if available
              if (textareaId && typeof Drupal.CKEditor5Instances !== 'undefined' && Drupal.CKEditor5Instances[textareaId]) {
                try {
                  value = Drupal.CKEditor5Instances[textareaId].getData();
                } catch (error) {
                  console.warn('Error getting CKEditor5 data:', error);
                  // Fall back to textarea value
                }
              }

              var format = $formatSelect.length ? $formatSelect.val() : 'basic_html';

              return {
                value: value,
                format: format,
                processed: value // For preview purposes, use the raw value
              };
            }
          }

          // Handle other field types (existing logic)
          if ($field.is(':checkbox')) {
            return $field.is(':checked');
          } else if ($field.is(':radio')) {
            return $field.is(':checked') ? $field.val() : NULL;
          } else if ($field.is('select[multiple]')) {
            return $field.val() || [];
          } else {
            var value = $field.val();
            // Try to parse JSON for arrays and objects
            if (typeof value === 'string' && (value.startsWith('[') || value.startsWith('{'))) {
              try {
                return JSON.parse(value);
              } catch (e) {
                return value;
              }
            }
            return value;
          }
        } catch (error) {
          console.error('Error getting field value:', error);
          return '';
        }
      }

      function extractPropName(fieldName) {
        if (!fieldName) { return NULL;
        }

        var matches = fieldName.match(/\[config\]\[([^\]]+)\]/) ||
                     fieldName.match(/\[config_wrapper\]\[config\]\[([^\]]+)\]/);
        return matches ? matches[1] : NULL;
      }

      function generatePreviewEnhanced(componentType, config, callback) {
        try {
          // Use the existing preview generation but with enhanced text handling
          var html = generateComponentHTMLEnhanced(componentType, config);
          callback(html);
        } catch (error) {
          console.error('Error generating preview:', error);
          callback('<div class="component-field-preview-error">Preview error</div>');
        }
      }

      function generateComponentHTMLEnhanced(componentType, config) {
        try {
          // Enhanced version that properly handles rich text content
          switch (componentType) {
            case 'button':
              return generateButtonHTMLEnhanced(config);

            case 'card':
              return generateCardHTMLEnhanced(config);

            case 'typography':
              return generateTypographyHTMLEnhanced(config);

            default:
              return generateGenericHTMLEnhanced(componentType, config);
          }
        } catch (error) {
          console.error('Error generating component HTML:', error);
          return '<div class="component-field-preview-error">Component preview error</div>';
        }
      }

      function generateButtonHTMLEnhanced(config) {
        var variant = config.variant || 'contained';
        var color = config.color || 'primary';
        var size = config.size || 'medium';
        var disabled = !!config.disabled;
        var label = config.label || 'Button';

        // Handle rich text label with enhanced processing
        if (typeof label === 'object' && label.processed) {
          label = label.processed;
        } else if (typeof label === 'object' && label.value) {
          label = label.value;
        }

        var classes = [
          'ComponentButton-root',
          'ComponentButtonBase-root',
          'ComponentButton-' + variant,
          'ComponentButton-' + variant + capitalize(color),
          'ComponentButton-size' + capitalize(size)
        ];

        if (disabled) { classes.push('Component-disabled');
        }

        var html = '<div class="component-field-preview-component">';
        html += '<button class="' + classes.join(' ') + '"';
        if (disabled) { html += ' disabled';
        }
        html += '>';
        html += '<span class="ComponentButton-label">' + label + '</span>';
        html += '</button></div>';

        return html;
      }

      function generateCardHTMLEnhanced(config) {
        var elevation = config.elevation || 1;
        var variant = config.variant || 'elevation';
        var title = config.title || '';
        var content = config.content || '';

        // Handle rich text content with enhanced processing
        if (typeof title === 'object' && title.processed) {
          title = title.processed;
        } else if (typeof title === 'object' && title.value) {
          title = title.value;
        }

        if (typeof content === 'object' && content.processed) {
          content = content.processed;
        } else if (typeof content === 'object' && content.value) {
          content = content.value;
        }

        var classes = [
          'ComponentCard-root',
          'ComponentPaper-root',
          variant === 'outlined' ? 'ComponentPaper-outlined' : 'ComponentPaper-elevation' + elevation
        ];

        var html = '<div class="component-field-preview-component">';
        html += '<div class="' + classes.join(' ') + '" style="max-width: 300px;">';

        if (title) {
          html += '<div class="ComponentCardHeader-root">';
          html += '<h6 class="ComponentCardHeader-title">' + title + '</h6>';
          html += '</div>';
        }

        if (content) {
          html += '<div class="ComponentCardContent-root">';
          html += '<div>' + content + '</div>';
          html += '</div>';
        }

        html += '</div></div>';
        return html;
      }

      function generateTypographyHTMLEnhanced(config) {
        var variant = config.variant || 'body1';
        var content = config.content || config.text || 'Sample text';

        // Handle rich text content with enhanced processing
        if (typeof content === 'object' && content.processed) {
          content = content.processed;
        } else if (typeof content === 'object' && content.value) {
          content = content.value;
        }

        var classes = ['ComponentTypography-root', 'ComponentTypography-' + variant];

        var html = '<div class="component-field-preview-component">';
        html += '<div class="' + classes.join(' ') + '">' + content + '</div>';
        html += '</div>';

        return html;
      }

      function generateGenericHTMLEnhanced(componentType, config) {
        var html = '<div class="component-field-preview-component component-field-preview-generic">';
        html += '<div class="component-field-preview-header">📦 ' + escapeHtml(componentType) + '</div>';

        if (Object.keys(config).length > 0) {
          html += '<div class="component-field-preview-sample">';
          html += '<p>Component configured with:</p>';
          html += '<ul>';

          Object.keys(config).slice(0, 3).forEach(function (key) {
            var value = config[key];
            var displayValue;

            if (typeof value === 'object' && value.processed) {
              displayValue = 'Rich Text Content';
            } else if (typeof value === 'object' && value.value) {
              displayValue = 'Text: ' + String(value.value).substring(0, 20) + '...';
            } else if (Array.isArray(value)) {
              displayValue = 'Array (' + value.length + ' items)';
            } else {
              displayValue = String(value);
              if (displayValue.length > 30) {
                displayValue = displayValue.substring(0, 27) + '...';
              }
            }

            html += '<li><code>' + escapeHtml(key) + '</code>: ' + escapeHtml(displayValue) + '</li>';
          });

          if (Object.keys(config).length > 3) {
            html += '<li><em>... and ' + (Object.keys(config).length - 3) + ' more</em></li>';
          }

          html += '</ul></div>';
        }

        html += '</div>';
        return html;
      }

      function capitalize(str) {
        return str.charAt(0).toUpperCase() + str.slice(1);
      }

      function escapeHtml(text) {
        var div = document.createElement('div');
        div.textContent = text;
        return div.innerHTML;
      }
    }
  };

  // Global event listener for CKEditor5 events
  $(document).ready(function () {
    // Listen for CKEditor5 ready events
    $(document).on('CKEditor5:ready', function (event, editor) {
      // Handle CKEditor5 initialization
      console.log('CKEditor5 ready for Component Field');
    });
  });

})(jQuery, Drupal, drupalSettings, once);