(function ($, Drupal, drupalSettings) {
  'use strict';

  /**
   * Enhanced Component Field Renderer
   */
  Drupal.behaviors.componentFieldRenderer = {
    attach: function (context, settings) {
      if (!settings.componentField) {
        return;
      }

      // Process dynamic components
      $('.component-field-dynamic', context).once('component-field-renderer').each(function () {
        var $element = $(this);
        renderDynamicComponent($element);
      });

      // Process static SDC components
      $('.component-field-fielded', context).once('component-field-static').each(function () {
        var $element = $(this);
        enhanceStaticComponent($element);
      });

      function renderDynamicComponent($element) {
        var componentType = $element.data('component-type');
        var configuration = $element.data('configuration');

        if (!componentType || !configuration) {
          return;
        }

        // Add component metadata
        $element.addClass('component-field-' + componentType.toLowerCase());
        $element.attr('data-component-field-component', componentType);

        // Apply configuration as data attributes for theming
        Object.keys(configuration).forEach(function (key) {
          $element.attr('data-component-field-' + key.toLowerCase(), configuration[key]);
        });

        // Trigger component-specific behaviors
        $element.trigger('componentField:component:rendered', {
          type: componentType,
          config: configuration
        });
      }

      function enhanceStaticComponent($element) {
        var componentType = $element.data('component-type');

        if (!componentType) {
          return;
        }

        // Add common enhancements
        $element.addClass('component-field-enhanced');

        // Component-specific enhancements
        switch (componentType) {
          case 'button':
            enhanceButton($element);
            break;

          case 'card':
            enhanceCard($element);
            break;

          default:
            // Generic enhancement
            break;
        }
      }

      function enhanceButton($button) {
        // Add ripple effect if not already present
        if (!$button.find('.ComponentTouchRipple-root').length) {
          $button.append('<span class="ComponentTouchRipple-root"></span>');
        }

        // Button click handler
        $button.on('click', function (e) {
          if ($button.hasClass('Component-disabled')) {
            e.preventDefault();
            return FALSE;
          }

          createRippleEffect(e, $button);
        });
      }

      function enhanceCard($card) {
        // Add hover effects
        $card.on('mouseenter', function () {
          $(this).addClass('component-field-card-hover');
        }).on('mouseleave', function () {
          $(this).removeClass('component-field-card-hover');
        });
      }

      function createRippleEffect(event, $element) {
        var $rippleContainer = $element.find('.ComponentTouchRipple-root');
        if (!$rippleContainer.length) { return;
        }

        var rect = $element[0].getBoundingClientRect();
        var size = Math.max(rect.width, rect.height);
        var x = event.clientX - rect.left - size / 2;
        var y = event.clientY - rect.top - size / 2;

        var $ripple = $('<span class="ComponentTouchRipple-ripple"></span>');
        $ripple.css({
          position: 'absolute',
          borderRadius: '50%',
          backgroundColor: 'currentColor',
          opacity: 0.3,
          width: size,
          height: size,
          left: x,
          top: y,
          pointerEvents: 'none',
          transform: 'scale(0)',
          transition: 'transform 600ms cubic-bezier(0.4, 0, 0.2, 1)'
        });

        $rippleContainer.append($ripple);

        // Trigger animation
        setTimeout(function () {
          $ripple.css('transform', 'scale(1)');
        }, 0);

        // Remove after animation
        setTimeout(function () {
          $ripple.remove();
        }, 600);
      }

      // Global Component Field utilities
      window.ComponentFields = window.ComponentFields || {
        render: renderDynamicComponent,
        enhance: enhanceStaticComponent,
        ripple: createRippleEffect
      };
    }
  };

})(jQuery, Drupal, drupalSettings);