(function ($, Drupal, drupalSettings) {
  'use strict';

  /**
   * Enhanced Component Field Widget behavior with improved preview rendering
   */
  Drupal.behaviors.componentFieldWidget = {
    attach: function (context, settings) {
      // Initialize widget containers
      $('.component-field-widget', context).once('component-field-widget').each(function () {
        var $widget = $(this);
        initializeWidget($widget);
      });

      // Initialize component items (handle dynamic content)
      $('.component-field-item', context).each(function () {
        var $item = $(this);
        if (!$item.hasClass('js-component-field-processed')) {
          $item.addClass('js-component-field-processed');
          initializeComponentItem($item);
        }
      });

      function initializeWidget($widget) {
        // Add global event handlers with namespace to prevent conflicts
        $(document).off('change.componentField keyup.componentField')
          .on('change.componentField keyup.componentField', '.component-field-config-field', function () {
            var $field = $(this);
            var $item = $field.closest('.component-field-item');
            if ($item.length) {
              updateComponentPreview($item);
            }
          });

        // Handle component type changes with delayed preview update
        $(document).off('change.componentFieldType')
          .on('change.componentFieldType', 'select[name*="component_type"]', function () {
            var $select = $(this);
            var $item = $select.closest('.component-field-item');

            // Show loading state immediately
            $item.find('.component-field-preview-content').html(
              '<div class="component-field-preview-empty">' +
              '<span class="component-field-preview-icon">⟳</span>' +
              '<p>Loading component...</p>' +
              '</div>'
            );

            // Update preview after AJAX completes
            setTimeout(function () {
              updateComponentPreview($item);
            }, 500);
          });

        // Initialize sortable if available
        var $container = $widget.find('.component-field-items-container');
        if ($container.length && $.fn.sortable && !$container.hasClass('ui-sortable')) {
          $container.sortable({
            items: '> .component-field-item-wrapper',
            handle: '.component-field-item-header',
            placeholder: 'component-field-item-placeholder',
            tolerance: 'pointer',
            start: function (e, ui) {
              ui.item.addClass('ui-sortable-helper');
              ui.placeholder.html('<div class="component-field-sortable-placeholder">Drop component here</div>');
            },
            stop: function (e, ui) {
              ui.item.removeClass('ui-sortable-helper');
              updateComponentOrder($container);
            }
          });
        }
      }

      function initializeComponentItem($item) {
        // Remove component functionality
        $item.off('click.componentFieldRemove')
          .on('click.componentFieldRemove', '.component-field-remove-btn', function (e) {
            e.preventDefault();
            removeComponent($item);
          });

        // Toggle component details
        $item.off('click.componentFieldToggle')
          .on('click.componentFieldToggle', '.component-field-toggle-btn', function (e) {
            e.preventDefault();
            toggleItemCollapse($item);
          });

        // Initial preview update with delay to ensure DOM is ready
        setTimeout(function () {
          updateComponentPreview($item);
        }, 100);
      }

      function removeComponent($item) {
        if (confirm(Drupal.t('Are you sure you want to remove this component?'))) {
          $item.addClass('removing');

          // Find and trigger the Drupal remove button
          var $removeBtn = $item.find('input[name*="_remove"]');
          if ($removeBtn.length) {
            $removeBtn.trigger('mousedown');
          } else {
            // If no Drupal remove button, just hide the item
            $item.fadeOut(300, function () {
              $(this).remove();
            });
          }
        }
      }

      function toggleItemCollapse($item) {
        var $content = $item.find('.component-field-item-content');
        var $btn = $item.find('.component-field-toggle-btn');

        $content.slideToggle(300);
        $item.toggleClass('component-field-item-collapsed');

        var isCollapsed = $item.hasClass('component-field-item-collapsed');
        $btn.text(isCollapsed ? '📂 Expand' : '📁 Collapse');
      }

      function updateComponentPreview($item) {
        var $preview = $item.find('.component-field-preview-content');
        if (!$preview.length) {
          return;
        }

        var componentType = $item.find('select[name*="component_type"]').val();
        if (!componentType) {
          $preview.html(
            '<div class="component-field-preview-empty">' +
            '<span class="component-field-preview-icon">🎨</span>' +
            '<p>Select a component to see preview</p>' +
            '</div>'
          );
          return;
        }

        // Collect configuration from config fields
        var config = {};
        $item.find('.component-field-config-field').each(function () {
          var $field = $(this);
          var propName = $field.data('prop-name') || extractPropName($field.attr('name'));

          if (propName) {
            var value = getFieldValue($field);
            if (value !== NULL && value !== '') {
              config[propName] = value;
            }
          }
        });

        // Generate and display preview
        generatePreview(componentType, config, function (html) {
          $preview.html(html);
        });
      }

      function extractPropName(fieldName) {
        if (!fieldName) { return NULL;
        }

        // Extract prop name from field name patterns
        var matches = fieldName.match(/\[config\]\[([^\]]+)\]/) ||
                     fieldName.match(/\[config_wrapper\]\[config\]\[([^\]]+)\]/);
        return matches ? matches[1] : NULL;
      }

      function getFieldValue($field) {
        if ($field.is(':checkbox')) {
          return $field.is(':checked');
        } else if ($field.is(':radio')) {
          return $field.is(':checked') ? $field.val() : NULL;
        } else if ($field.is('select[multiple]')) {
          return $field.val() || [];
        } else {
          var value = $field.val();
          // Try to parse JSON for arrays and objects
          if (typeof value === 'string' && (value.startsWith('[') || value.startsWith('{'))) {
            try {
              return JSON.parse(value);
            } catch (e) {
              return value;
            }
          }
          return value;
        }
      }

      function generatePreview(componentType, config, callback) {
        // Apply defaults from component info if available
        var componentInfo = drupalSettings.componentField &&
                           drupalSettings.componentField.components &&
                           drupalSettings.componentField.components[componentType];

        if (componentInfo && componentInfo.props) {
          Object.keys(componentInfo.props).forEach(function (propName) {
            var propSchema = componentInfo.props[propName];
            if (!config.hasOwnProperty(propName) && propSchema.hasOwnProperty('default')) {
              config[propName] = propSchema.default;
            }
          });
        }

        var html = generateComponentHTML(componentType, config, componentInfo);
        callback(html);
      }

      function generateComponentHTML(componentType, config, componentInfo) {
        switch (componentType) {
          case 'button':
            return generateButtonHTML(config);

          case 'card':
            return generateCardHTML(config);

          case 'typography':
            return generateTypographyHTML(config);

          case 'test':
            return generateTestHTML(config);

          default:
            return generateGenericHTML(componentType, config, componentInfo);
        }
      }

      function generateButtonHTML(config) {
        var variant = config.variant || 'contained';
        var color = config.color || 'primary';
        var size = config.size || 'medium';
        var disabled = !!config.disabled;
        var fullWidth = !!config.fullWidth;
        var label = config.label || 'Button';

        var classes = [
          'ComponentButton-root',
          'ComponentButtonBase-root',
          'ComponentButton-' + variant,
          'ComponentButton-' + variant + capitalize(color),
          'ComponentButton-size' + capitalize(size)
        ];

        if (disabled) { classes.push('Component-disabled');
        }
        if (fullWidth) { classes.push('ComponentButton-fullWidth');
        }

        var html = '<div class="component-field-preview-component">';
        html += '<button class="' + classes.join(' ') + '"';
        if (disabled) { html += ' disabled';
        }
        html += '>';

        if (config.startIcon) {
          html += '<span class="ComponentButton-startIcon">' + escapeHtml(config.startIcon) + '</span>';
        }

        html += '<span class="ComponentButton-label">' + escapeHtml(label) + '</span>';

        if (config.endIcon) {
          html += '<span class="ComponentButton-endIcon">' + escapeHtml(config.endIcon) + '</span>';
        }

        html += '</button></div>';
        return html;
      }

      function generateCardHTML(config) {
        var elevation = config.elevation || 1;
        var variant = config.variant || 'elevation';
        var square = !!config.square;

        var classes = [
          'ComponentCard-root',
          'ComponentPaper-root',
          variant === 'outlined' ? 'ComponentPaper-outlined' : 'ComponentPaper-elevation' + elevation,
          square ? 'ComponentPaper-square' : 'ComponentPaper-rounded'
        ];

        var html = '<div class="component-field-preview-component">';
        html += '<div class="' + classes.join(' ') + '">';

        // Card header
        if (config.title || config.subheader) {
          html += '<div class="ComponentCardHeader-root">';
          if (config.title) {
            html += '<h6 class="ComponentCardHeader-title">' + escapeHtml(config.title) + '</h6>';
          }
          if (config.subheader) {
            html += '<div class="ComponentCardHeader-subheader">' + escapeHtml(config.subheader) + '</div>';
          }
          html += '</div>';
        }

        // Card content
        var content = config.content || 'This is sample card content to show how your card will look.';
        html += '<div class="ComponentCardContent-root">';
        html += '<p>' + escapeHtml(content) + '</p>';
        html += '</div>';

        // Card actions
        if (config.actions) {
          html += '<div class="ComponentCardActions-root">';
          html += escapeHtml(config.actions);
          html += '</div>';
        }

        html += '</div></div>';
        return html;
      }

      function generateTypographyHTML(config) {
        var variant = config.variant || 'body1';
        var component = config.component || getDefaultComponent(variant);
        var color = config.color || 'inherit';
        var align = config.align || 'inherit';
        var gutterBottom = !!config.gutterBottom;
        var noWrap = !!config.noWrap;

        var classes = ['ComponentTypography-root', 'ComponentTypography-' + variant];
        if (color !== 'inherit') { classes.push('ComponentTypography-color' + capitalize(color));
        }
        if (align !== 'inherit') { classes.push('ComponentTypography-align' + capitalize(align));
        }
        if (gutterBottom) { classes.push('ComponentTypography-gutterBottom');
        }
        if (noWrap) { classes.push('ComponentTypography-noWrap');
        }

        var text = config.children || config.text || getDefaultText(variant);

        var html = '<div class="component-field-preview-component">';
        html += '<' + component + ' class="' + classes.join(' ') + '">';
        html += escapeHtml(text);
        html += '</' + component + '>';
        html += '</div>';

        return html;
      }

      function generateTestHTML(config) {
        var variant = config.variant || 'primary';
        var size = config.size || 'medium';
        var disabled = !!config.disabled;
        var title = config.title || 'Test Component';
        var content = config.content || 'This is a test of the Component Field system.';
        var counter = config.counter || 0;

        var classes = [
          'ComponentTest-root',
          'ComponentTest-' + variant,
          'ComponentTest-size' + capitalize(size)
        ];
        if (disabled) { classes.push('ComponentTest-disabled');
        }

        var html = '<div class="component-field-preview-component">';
        html += '<div class="' + classes.join(' ') + '">';

        // Header
        html += '<div class="ComponentTest-header">';
        html += '<h4 class="ComponentTest-title">' + escapeHtml(title) + '</h4>';
        html += '<div class="ComponentTest-meta">';
        html += '<span class="ComponentTest-variant">' + escapeHtml(variant) + '</span>';
        html += '<span class="ComponentTest-size">' + escapeHtml(size) + '</span>';
        html += '</div></div>';

        // Content
        html += '<div class="ComponentTest-content">';
        html += '<div class="ComponentTest-description">' + escapeHtml(content) + '</div>';

        if (counter !== NULL && counter !== undefined) {
          html += '<div class="ComponentTest-counter"><strong>Counter:</strong> ' + parseInt(counter) + '</div>';
        }

        html += '<div class="ComponentTest-status">';
        html += '<span>Status:</span>';
        html += '<span class="ComponentTest-statusBadge ' + (disabled ? 'disabled' : 'enabled') + '">';
        html += disabled ? 'Disabled' : 'Enabled';
        html += '</span></div></div>';

        // Footer
        html += '<div class="ComponentTest-footer">✅ Component Field Test Working!</div>';
        html += '</div></div>';

        return html;
      }

      function generateGenericHTML(componentType, config, componentInfo) {
        var label = (componentInfo && componentInfo.label) || componentType;

        var html = '<div class="component-field-preview-component component-field-preview-generic">';
        html += '<div class="component-field-preview-header">📦 ' + escapeHtml(label) + '</div>';

        if (Object.keys(config).length > 0) {
          html += '<div class="component-field-preview-sample">';
          html += '<p>Component configured with:</p>';
          html += '<ul>';

          var configKeys = Object.keys(config).slice(0, 3);
          configKeys.forEach(function (key) {
            var value = config[key];
            var displayValue = Array.isArray(value) ? 'Array' : String(value);
            if (displayValue.length > 30) {
              displayValue = displayValue.substring(0, 27) + '...';
            }
            html += '<li><code>' + escapeHtml(key) + '</code>: ' + escapeHtml(displayValue) + '</li>';
          });

          if (Object.keys(config).length > 3) {
            html += '<li><em>... and ' + (Object.keys(config).length - 3) + ' more</em></li>';
          }

          html += '</ul></div>';
        } else {
          html += '<div class="component-field-preview-sample"><p>No configuration set</p></div>';
        }

        html += '</div>';
        return html;
      }

      function getDefaultComponent(variant) {
        var mapping = {
          h1: 'h1', h2: 'h2', h3: 'h3', h4: 'h4', h5: 'h5', h6: 'h6',
          subtitle1: 'h6', subtitle2: 'h6',
          body1: 'p', body2: 'p',
          caption: 'span', button: 'span', overline: 'span'
        };
        return mapping[variant] || 'p';
      }

      function getDefaultText(variant) {
        var texts = {
          h1: 'Heading 1', h2: 'Heading 2', h3: 'Heading 3',
          h4: 'Heading 4', h5: 'Heading 5', h6: 'Heading 6',
          subtitle1: 'Subtitle 1', subtitle2: 'Subtitle 2',
          body1: 'Body text with regular weight and size for comfortable reading.',
          body2: 'Body text with smaller size.',
          caption: 'Caption text', button: 'BUTTON TEXT', overline: 'OVERLINE TEXT'
        };
        return texts[variant] || 'Sample text';
      }

      function capitalize(str) {
        return str.charAt(0).toUpperCase() + str.slice(1);
      }

      function escapeHtml(text) {
        var div = document.createElement('div');
        div.textContent = text;
        return div.innerHTML;
      }

      function updateComponentOrder($container) {
        $container.find('.component-field-item').each(function (index) {
          $(this).find('input[name*="[_weight]"]').val(index);
        });
      }

      // Handle AJAX completion - crucial for new components
      $(document).off('ajaxComplete.componentField')
        .on('ajaxComplete.componentField', function (event, xhr, settings) {
          // Re-run behaviors on new content after AJAX
          setTimeout(function () {
            Drupal.behaviors.componentFieldWidget.attach(document, drupalSettings);
          }, 100);
        });

      // Handle successful AJAX updates
      $(document).off('ajaxSuccess.componentField')
        .on('ajaxSuccess.componentField', function (event, xhr, settings) {
          // Update previews after successful AJAX
          setTimeout(function () {
            $('.component-field-item').each(function () {
              updateComponentPreview($(this));
            });
          }, 200);
        });

      // Handle Drupal AJAX commands
      $(document).off('drupalAjaxCommandResponse.componentField')
        .on('drupalAjaxCommandResponse.componentField', function (event, xhr, settings) {
          // Refresh previews after Drupal AJAX commands
          setTimeout(function () {
            $('.component-field-item:not(.js-component-field-processed)').each(function () {
              initializeComponentItem($(this));
            });
          }, 300);
        });
    }
  };

})(jQuery, Drupal, drupalSettings);