<?php

namespace Drupal\component_field\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\component_field\Service\ComponentDiscovery;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Enhanced Controller for Component Field administration.
 */
class ComponentFieldAdminController extends ControllerBase {

  use DependencySerializationTrait;

  /**
   * The component discovery service.
   */
  protected ComponentDiscovery $componentDiscovery;

  /**
   * Constructor.
   */
  public function __construct(ComponentDiscovery $component_discovery) {
    $this->componentDiscovery = $component_discovery;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('component_field.discovery')
    );
  }

  /**
   * Overview page for discovered components.
   */
  public function overview() {
    $components = $this->componentDiscovery->discoverComponents();

    $header = [
      $this->t('Component'),
      $this->t('Type'),
      $this->t('Source File'),
      $this->t('Props'),
      $this->t('Last Modified'),
      $this->t('Operations'),
    ];

    $rows = [];
    foreach ($components as $name => $component) {
      $operations = [
        '#type' => 'operations',
        '#links' => [],
      ];

      $rows[] = [
        'data' => [
          [
            'data' => [
              '#markup' => '<strong>' . $component['label'] . '</strong><br><small>' . $name . '</small>',
            ],
          ],
          'SDC',
          [
            'data' => [
              '#markup' => '<code>' . $component['relative_path'] . '</code>',
            ],
          ],
          count($component['props']),
          date('Y-m-d H:i:s', $component['modified']),
          [
            'data' => $operations,
          ],
        ],
        'class' => ['component-field-admin-component-item'],
        'data-component-name' => $name,
      ];
    }

    $build = [];

    $build['actions'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['component-field-overview-actions']],
    ];

    $build['actions']['refresh'] = [
      '#type' => 'link',
      '#title' => $this->t('🔄 Refresh Components'),
      '#url' => Url::fromRoute('component_field.admin.refresh'),
      '#attributes' => [
        'class' => ['button', 'button--primary'],
      ],
    ];

    $build['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No Component Field components discovered. Check your theme components directory.'),
      '#attributes' => [
        'class' => ['component-field-overview'],
      ],
    ];

    $build['#attached']['library'][] = 'component_field/admin';

    return $build;
  }

  /**
   * Refresh component discovery cache.
   */
  public function refresh() {
    $this->componentDiscovery->clearCache();
    $components = $this->componentDiscovery->discoverComponents(TRUE);

    $this->messenger()->addStatus(
      $this->t('✅ Component discovery refreshed. Found @count components.', [
        '@count' => count($components),
      ])
    );

    return $this->redirect('component_field.admin.overview');
  }

  /**
   * AJAX endpoint for component preview.
   */
  public function ajaxPreview(Request $request) {
    $component_type = $request->get('component_type');
    $config = $request->get('config', []);

    if (empty($component_type)) {
      return new JsonResponse([
        'error' => 'Component type is required',
      ], 400);
    }

    $components = $this->componentDiscovery->discoverComponents();
    if (!isset($components[$component_type])) {
      return new JsonResponse([
        'error' => 'Component not found',
      ], 404);
    }

    $component_info = $components[$component_type];
    $preview_html = $this->generatePreviewHtml($component_type, $config, $component_info);

    if ($request->get('ajax')) {
      $response = new AjaxResponse();
      $response->addCommand(new HtmlCommand('.component-field-preview-content', $preview_html));
      return $response;
    }

    return new JsonResponse([
      'preview_html' => $preview_html,
      'component_info' => [
        'name' => $component_info['label'],
        'props_count' => count($component_info['props']),
      ],
    ]);
  }

  /**
   * Generate preview HTML for a component.
   */
  protected function generatePreviewHtml(string $component_type, array $config, array $component_info): string {
    $html = '<div class="component-field-preview-component">';

    switch ($component_type) {
      case 'button':
        $variant = $config['variant'] ?? 'contained';
        $color = $config['color'] ?? 'primary';
        $html .= '<button class="ComponentButton-root ComponentButton-' . $variant . ' ComponentButton-color-' . $color . '">';
        $html .= htmlspecialchars($config['label'] ?? 'Button');
        $html .= '</button>';
        break;

      case 'card':
        $elevation = $config['elevation'] ?? 1;
        $html .= '<div class="ComponentCard-root ComponentPaper-elevation' . $elevation . '">';
        if (!empty($config['title'])) {
          $html .= '<div class="ComponentCardHeader-root"><h5>' . htmlspecialchars($config['title']) . '</h5></div>';
        }
        if (!empty($config['content'])) {
          $html .= '<div class="ComponentCardContent-root"><p>' . htmlspecialchars($config['content']) . '</p></div>';
        }
        $html .= '</div>';
        break;

      default:
        $html .= '<div class="component-field-preview-placeholder">';
        $html .= '<strong>' . htmlspecialchars($component_info['label']) . '</strong><br>';
        $html .= '<small>Component preview</small>';
        if (!empty($config)) {
          $html .= '<pre><code>' . json_encode($config, JSON_PRETTY_PRINT) . '</code></pre>';
        }
        $html .= '</div>';
    }

    $html .= '</div>';
    return $html;
  }

}
