<?php

namespace Drupal\component_field\Plugin\Field\FieldFormatter;

use Drupal\Core\Template\Attribute;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\component_field\Service\ComponentDiscovery;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Enhanced Plugin implementation of the 'component_field_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "component_field_formatter",
 *   label = @Translation("Component Field (Enhanced SDC)"),
 *   field_types = {
 *     "component_field"
 *   }
 * )
 */
class ComponentFieldFormatter extends FormatterBase implements ContainerFactoryPluginInterface {

  use DependencySerializationTrait;

  /**
   * The component discovery service.
   */
  protected ComponentDiscovery $componentDiscovery;

  /**
   * Constructor.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    ComponentDiscovery $component_discovery,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->componentDiscovery = $component_discovery;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('component_field.discovery')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $components = $this->componentDiscovery->discoverComponents();

    foreach ($items as $delta => $item) {
      $component_type = $item->get('component_type')->getValue();
      $configuration = $item->getConfiguration();

      if (empty($component_type)) {
        continue;
      }

      if (!isset($components[$component_type])) {
        // Component not found - show fallback if enabled.
        if ($this->getSetting('enable_fallback')) {
          $elements[$delta] = $this->buildFallbackComponent($component_type, $configuration, $delta);
        }
        else {
          \Drupal::logger('component_field')->warning('Component @type not found', ['@type' => $component_type]);
        }
        continue;
      }

      $component_info = $components[$component_type];

      // Try SDC first, fall back to custom rendering.
      $elements[$delta] = $this->tryRenderSdcComponent($component_type, $configuration, $component_info, $delta)
        ?: $this->buildCustomComponent($component_type, $configuration, $component_info, $delta);
    }

    // Add field wrapper if requested.
    if ($this->getSetting('add_wrapper') && !empty($elements)) {
      return [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['component-field-components', 'component-field-components-' . $this->fieldDefinition->getName()],
        ],
        'items' => $elements,
      ];
    }

    return $elements;
  }

  /**
   * Try to render using Drupal's SDC system.
   */
  protected function tryRenderSdcComponent(string $component_type, array $config, array $component_info, int $delta): ?array {
    try {
      // Check if SDC is available and component is registered.
      if (!\Drupal::hasService('plugin.manager.sdc')) {
        return NULL;
      }

      $sdc_manager = \Drupal::service('plugin.manager.sdc');

      // Try to find the component in SDC registry.
      $sdc_definitions = $sdc_manager->getDefinitions();
      $sdc_component_id = NULL;

      // Look for component by various possible IDs.
      $possible_ids = [
        $component_type,
        'component_field:' . $component_type,
        'component_field_base:' . $component_type,
      ];

      foreach ($possible_ids as $id) {
        if (isset($sdc_definitions[$id])) {
          $sdc_component_id = $id;
          break;
        }
      }

      if (!$sdc_component_id) {
        \Drupal::logger('component_field')->debug('SDC component @type not found in registry', ['@type' => $component_type]);
        return NULL;
      }

      // Validate and filter configuration.
      $validated_config = $this->validateConfiguration($config, $component_info);

      $build = [
        '#type' => 'component',
        '#component' => $sdc_component_id,
        '#props' => $validated_config,
      ];

      // Add wrapper if requested.
      if ($this->getSetting('add_item_wrapper')) {
        $build = $this->wrapComponent($build, $component_type, $delta);
      }

      // Add library dependencies.
      $this->attachLibraries($build, $component_info, $component_type);

      return $build;

    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('SDC rendering failed for @type: @message', [
        '@type' => $component_type,
        '@message' => $e->getMessage(),
      ]);
      return NULL;
    }
  }

  /**
   * Build custom component when SDC fails.
   */
  protected function buildCustomComponent(string $component_type, array $config, array $component_info, int $delta): array {
    $validated_config = $this->validateConfiguration($config, $component_info);

    // Use Twig template directly if available.
    if (!empty($component_info['template_file']) && file_exists($component_info['template_file'])) {
      $build = [
        '#type' => 'inline_template',
        '#template' => file_get_contents($component_info['template_file']),
        '#context' => array_merge($validated_config, [
          'attributes' => new Attribute(),
          'children' => '',
        ]),
      ];
    }
    else {
      // Generate HTML directly.
      $build = [
        '#markup' => $this->generateComponentHtml($component_type, $validated_config),
      ];
    }

    // Add wrapper if requested.
    if ($this->getSetting('add_item_wrapper')) {
      $build = $this->wrapComponent($build, $component_type, $delta);
    }

    // Add library dependencies.
    $this->attachLibraries($build, $component_info, $component_type);

    return $build;
  }

  /**
   * Wrap component with container.
   */
  protected function wrapComponent(array $build, string $component_type, int $delta): array {
    $wrapper_classes = [
      'component-field-wrapper',
      'component-field-' . str_replace(['_', '-'], '-', $component_type),
      'component-field-delta-' . $delta,
    ];

    if ($this->getSetting('show_animation')) {
      $wrapper_classes[] = 'component-field-animated';
    }

    return [
      '#type' => 'container',
      '#attributes' => [
        'class' => $wrapper_classes,
        'data-component-type' => $component_type,
        'data-component-delta' => $delta,
      ],
      'component' => $build,
    ];
  }

  /**
   * Attach component libraries.
   */
  protected function attachLibraries(array &$build, array $component_info, string $component_type): void {
    // Add theme library.
    $theme_name = \Drupal::theme()->getActiveTheme()->getName();
    $library_name = str_replace('_', '-', $component_type);
    $build['#attached']['library'][] = "{$theme_name}/{$library_name}";

    // Add base library.
    $build['#attached']['library'][] = "{$theme_name}/component-field-base";

    // Add component-specific library from discovery.
    if (!empty($component_info['metadata']['libraryDependencies'])) {
      foreach ($component_info['metadata']['libraryDependencies'] as $library) {
        $build['#attached']['library'][] = $library;
      }
    }

    // Add debug information if enabled.
    if ($this->getSetting('show_debug_info') && \Drupal::currentUser()->hasPermission('administer component field')) {
      $build['#attached']['drupalSettings']['componentField']['debug'][$component_type] = [
        'type' => $component_type,
        'config' => $this->validateConfiguration([], $component_info),
        'source' => $component_info['relative_path'],
      ];
    }
  }

  /**
   * Generate HTML for components that can't use templates.
   */
  protected function generateComponentHtml(string $component_type, array $config): string {
    switch ($component_type) {
      case 'test':
        return $this->generateTestHtml($config);

      case 'button':
        return $this->generateButtonHtml($config);

      case 'card':
        return $this->generateCardHtml($config);

      default:
        return $this->generateGenericHtml($component_type, $config);
    }
  }

  /**
   * Generate HTML for test component.
   */
  protected function generateTestHtml(array $config): string {
    $variant = $config['variant'] ?? 'primary';
    $size = $config['size'] ?? 'medium';
    $disabled = !empty($config['disabled']);
    $title = $config['title'] ?? 'Test Component';
    $content = $config['content'] ?? 'This is a test of the Component Field system.';

    $classes = [
      'ComponentTest-root',
      'ComponentTest-' . $variant,
      'ComponentTest-size' . ucfirst($size),
    ];
    if ($disabled) {
      $classes[] = 'ComponentTest-disabled';
    }

    $html = '<div class="' . implode(' ', $classes) . '">';
    $html .= '<div class="ComponentTest-header">';
    $html .= '<h4 class="ComponentTest-title">' . htmlspecialchars($title) . '</h4>';
    $html .= '</div>';
    $html .= '<div class="ComponentTest-content">';
    $html .= '<div class="ComponentTest-description">' . htmlspecialchars($content) . '</div>';
    $html .= '</div>';
    $html .= '<div class="ComponentTest-footer">✅ Component Field Test Working!</div>';
    $html .= '</div>';

    return $html;
  }

  /**
   * Generate HTML for button component.
   */
  protected function generateButtonHtml(array $config): string {
    $variant = $config['variant'] ?? 'contained';
    $color = $config['color'] ?? 'primary';
    $label = $config['label'] ?? 'Button';
    $disabled = !empty($config['disabled']);

    $classes = [
      'ComponentButton-root',
      'ComponentButtonBase-root',
      'ComponentButton-' . $variant,
      'ComponentButton-' . $variant . ucfirst($color),
    ];
    if ($disabled) {
      $classes[] = 'Component-disabled';
    }

    $html = '<button class="' . implode(' ', $classes) . '"';
    if ($disabled) {
      $html .= ' disabled';
    }
    $html .= '>' . htmlspecialchars($label) . '</button>';

    return $html;
  }

  /**
   * Generate HTML for card component.
   */
  protected function generateCardHtml(array $config): string {
    $elevation = $config['elevation'] ?? 1;
    $variant = $config['variant'] ?? 'elevation';

    $classes = [
      'ComponentCard-root',
      'ComponentPaper-root',
      $variant === 'outlined' ? 'ComponentPaper-outlined' : 'ComponentPaper-elevation' . $elevation,
    ];

    $html = '<div class="' . implode(' ', $classes) . '">';

    if (!empty($config['title'])) {
      $html .= '<div class="ComponentCardHeader-root">';
      $html .= '<h6 class="ComponentCardHeader-title">' . htmlspecialchars($config['title']) . '</h6>';
      $html .= '</div>';
    }

    if (!empty($config['content'])) {
      $html .= '<div class="ComponentCardContent-root">';
      $html .= '<p>' . htmlspecialchars($config['content']) . '</p>';
      $html .= '</div>';
    }

    $html .= '</div>';

    return $html;
  }

  /**
   * Generate generic HTML for unknown components.
   */
  protected function generateGenericHtml(string $component_type, array $config): string {
    $html = '<div class="component-field-generic" data-component="' . htmlspecialchars($component_type) . '">';
    $html .= '<h4>Component: ' . htmlspecialchars($component_type) . '</h4>';
    if (!empty($config)) {
      $html .= '<pre>' . htmlspecialchars(json_encode($config, JSON_PRETTY_PRINT)) . '</pre>';
    }
    $html .= '</div>';

    return $html;
  }

  /**
   * Enhanced validate configuration with rich content processing.
   */
  protected function validateConfiguration(array $config, array $component_info): array {
    $validated = [];

    if (empty($component_info['props'])) {
      return $config;
    }

    foreach ($component_info['props'] as $prop_name => $prop_schema) {
      if (!isset($config[$prop_name])) {
        // Use default value if available.
        if (isset($prop_schema['default'])) {
          $validated[$prop_name] = $prop_schema['default'];
        }
        continue;
      }

      $value = $config[$prop_name];
      $type = $prop_schema['type'] ?? 'string';
      $format = $prop_schema['format'] ?? NULL;

      // Type casting and validation with rich content support.
      switch ($type) {
        case 'text':
          $validated[$prop_name] = $this->processTextValue($value, $format);
          break;

        case 'entity_reference':
          $validated[$prop_name] = $this->processEntityReference($value, $prop_schema);
          break;

        case 'boolean':
          $validated[$prop_name] = (bool) $value;
          break;

        case 'integer':
        case 'number':
          if (is_numeric($value)) {
            $validated[$prop_name] = $type === 'integer' ? (int) $value : (float) $value;
          }
          else {
            $validated[$prop_name] = $prop_schema['default'] ?? 0;
          }
          break;

        case 'array':
          if (is_string($value)) {
            $decoded = json_decode($value, TRUE);
            $validated[$prop_name] = $decoded ?? [];
          }
          else {
            $validated[$prop_name] = (array) $value;
          }
          break;

        case 'object':
          if (is_string($value)) {
            $decoded = json_decode($value, TRUE);
            $validated[$prop_name] = $decoded ?? new \stdClass();
          }
          else {
            $validated[$prop_name] = (object) $value;
          }
          break;

        case 'string':
        default:
          // Validate enum values.
          if (!empty($prop_schema['enum']) && !in_array($value, $prop_schema['enum'])) {
            $validated[$prop_name] = $prop_schema['default'] ?? $prop_schema['enum'][0] ?? '';
          }
          else {
            $validated[$prop_name] = (string) $value;
          }
          break;
      }
    }

    return $validated;
  }

  /**
   * Process entity reference values.
   */
  protected function processEntityReference($value, array $prop_schema) {
    $target_type = $prop_schema['target_type'] ?? 'node';

    // Handle array of entity IDs (from media library)
    if (is_array($value) && !isset($value['target_id'])) {
      $entities = [];
      foreach ($value as $id) {
        if ($entity = \Drupal::entityTypeManager()->getStorage($target_type)->load($id)) {
          $entities[] = $entity;
        }
      }
      return $entities;
    }

    // Handle single entity reference.
    if (is_array($value) && isset($value['target_id'])) {
      $entity_type = $value['entity_type'] ?? $target_type;
      return \Drupal::entityTypeManager()->getStorage($entity_type)->load($value['target_id']);
    }

    // Handle direct entity ID.
    if (is_numeric($value)) {
      return \Drupal::entityTypeManager()->getStorage($target_type)->load($value);
    }

    return NULL;
  }

  /**
   * Render entity reference content.
   */
  protected function renderEntityReference($entity_ref, string $view_mode = 'default'): ?string {
    if (!$entity_ref) {
      return NULL;
    }

    // Handle array of entities.
    if (is_array($entity_ref)) {
      $output = '';
      foreach ($entity_ref as $entity) {
        if ($entity && method_exists($entity, 'id')) {
          $view_builder = \Drupal::entityTypeManager()->getViewBuilder($entity->getEntityTypeId());
          $render_array = $view_builder->view($entity, $view_mode);
          $output .= \Drupal::service('renderer')->render($render_array);
        }
      }
      return $output;
    }

    // Handle single entity.
    if ($entity_ref && method_exists($entity_ref, 'id')) {
      $view_builder = \Drupal::entityTypeManager()->getViewBuilder($entity_ref->getEntityTypeId());
      $render_array = $view_builder->view($entity_ref, $view_mode);
      return \Drupal::service('renderer')->render($render_array);
    }

    return NULL;
  }

  /**
   * Render text content properly.
   */
  protected function renderTextContent($content): string {
    if (is_array($content)) {
      if (isset($content['#markup'])) {
        return $content['#markup'];
      }
      elseif (isset($content['#text'], $content['#format'])) {
        return check_markup($content['#text'], $content['#format']);
      }
      else {
        return \Drupal::service('renderer')->render($content);
      }
    }

    return is_string($content) ? htmlspecialchars($content) : '';
  }

  /**
   * Process text values with format support.
   */
  protected function processTextValue($value, ?string $format) {
    // Handle rich text format fields.
    if (is_array($value) && isset($value['value'], $value['format'])) {
      return [
        '#type' => 'processed_text',
        '#text' => $value['value'],
        '#format' => $value['format'],
      ];
    }

    // Handle processed content.
    if (is_array($value) && isset($value['processed'])) {
      return [
        '#markup' => $value['processed'],
      ];
    }

    // Handle raw text based on format.
    if (is_string($value)) {
      switch ($format) {
        case 'html':
        case 'raw_html':
          return ['#markup' => $value];

        case 'markdown':
          // If markdown module is available.
          if (\Drupal::moduleHandler()->moduleExists('markdown')) {
            return [
              '#type' => 'markdown',
              '#markdown' => $value,
            ];
          }
          return ['#markup' => nl2br(htmlspecialchars($value))];

        default:
          return ['#markup' => nl2br(htmlspecialchars($value))];
      }
    }

    return $value;
  }

  /**
   * Build fallback component when SDC component is not found.
   */
  protected function buildFallbackComponent(string $component_type, array $config, int $delta): array {
    return [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['component-field-fallback', 'component-field-missing'],
        'data-component-type' => $component_type,
      ],
      'content' => [
        '#markup' => '<div class="component-field-error">' .
        '<strong>' . $this->t('Component "@type" not found', ['@type' => $component_type]) . '</strong>' .
        (!empty($config) ? '<details><summary>Configuration</summary><pre>' . json_encode($config, JSON_PRETTY_PRINT) . '</pre></details>' : '') .
        '</div>',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'add_wrapper' => FALSE,
      'add_item_wrapper' => TRUE,
      'show_animation' => FALSE,
      'enable_fallback' => TRUE,
      'show_debug_info' => FALSE,
      'lazy_load' => FALSE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    $element['add_wrapper'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Add field wrapper'),
      '#default_value' => $this->getSetting('add_wrapper'),
      '#description' => $this->t('Wrap all components in a field-level container.'),
    ];

    $element['add_item_wrapper'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Add item wrappers'),
      '#default_value' => $this->getSetting('add_item_wrapper'),
      '#description' => $this->t('Wrap each component in an individual container.'),
    ];

    $element['show_animation'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable animations'),
      '#default_value' => $this->getSetting('show_animation'),
      '#description' => $this->t('Add CSS animation classes to components.'),
    ];

    $element['enable_fallback'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable fallback rendering'),
      '#default_value' => $this->getSetting('enable_fallback'),
      '#description' => $this->t('Show error message when components are not found.'),
    ];

    $element['show_debug_info'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show debug information'),
      '#default_value' => $this->getSetting('show_debug_info'),
      '#description' => $this->t('Add debug information to drupalSettings (admin users only).'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    $wrapper_setting = $this->getSetting('add_wrapper') ? 'Field + Item' :
                      ($this->getSetting('add_item_wrapper') ? 'Item only' : 'None');
    $summary[] = $this->t('Wrappers: @wrapper', ['@wrapper' => $wrapper_setting]);

    if ($this->getSetting('show_animation')) {
      $summary[] = $this->t('Animations: Enabled');
    }

    if ($this->getSetting('enable_fallback')) {
      $summary[] = $this->t('Fallback: Enabled');
    }

    $components = $this->componentDiscovery->discoverComponents();
    $summary[] = $this->t('Available components: @count', [
      '@count' => count($components),
    ]);

    return $summary;
  }

}
