<?php

namespace Drupal\component_field\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Defines the 'component_field' field type.
 *
 * @FieldType(
 *   id = "component_field",
 *   label = @Translation("Component Field"),
 *   description = @Translation("Auto-discovering component field"),
 *   default_widget = "component_field_widget",
 *   default_formatter = "component_field_formatter"
 * )
 */
class ComponentFieldItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['component_type'] = DataDefinition::create('string')
      ->setLabel(t('Component Type'))
      ->setRequired(TRUE);

    $properties['configuration'] = DataDefinition::create('string')
      ->setLabel(t('Component Configuration'))
      ->setDescription(t('JSON configuration for the component'));

    $properties['component_version'] = DataDefinition::create('string')
      ->setLabel(t('Component Version'))
      ->setDescription(t('Hash of component file for cache invalidation'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'component_type' => [
          'type' => 'varchar',
          'length' => 64,
          'not null' => TRUE,
        ],
        'configuration' => [
          'type' => 'text',
          'size' => 'big',
          'not null' => FALSE,
        ],
        'component_version' => [
          'type' => 'varchar',
          'length' => 32,
          'not null' => FALSE,
        ],
      ],
      'indexes' => [
        'component_type' => ['component_type'],
        'component_version' => ['component_version'],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    return empty($this->get('component_type')->getValue());
  }

  /**
   * Get the component configuration as array.
   */
  public function getConfiguration() {
    $config = $this->get('configuration')->getValue();
    return $config ? json_decode($config, TRUE) : [];
  }

  /**
   * Set the component configuration from array.
   */
  public function setConfiguration(array $config) {
    $this->set('configuration', json_encode($config));
  }

  /**
   * Get component version hash.
   */
  public function getComponentVersion() {
    return $this->get('component_version')->getValue();
  }

  /**
   * Set component version hash.
   */
  public function setComponentVersion(string $version) {
    $this->set('component_version', $version);
  }

}
