<?php

namespace Drupal\component_field\Plugin\Field\FieldType;

use Drupal\Core\TypedData\TypedData;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\ComputedItemListTrait;

/**
 * Computed field to extract searchable content from Component Field.
 *
 * @FieldType(
 *   id = "component_field_searchable_content",
 *   label = @Translation("Component Field Searchable Content"),
 *   description = @Translation("Computed field containing searchable content from Component Field"),
 *   default_widget = "string_textfield",
 *   default_formatter = "string",
 *   no_ui = TRUE,
 * )
 */
class ComponentFieldSearchableContent extends FieldItemBase {

  use ComputedItemListTrait;

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['value'] = DataDefinition::create('string')
      ->setLabel(t('Searchable Content'))
      ->setComputed(TRUE)
      ->setClass('\Drupal\component_field\TypedData\ComponentFieldSearchableContentValue');

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'value' => [
          'type' => 'text',
          'size' => 'big',
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    return empty($this->get('value')->getValue());
  }

}

/**
 * Computed property class for extracting searchable content.
 */
class ComponentFieldSearchableContentValue extends TypedData {

  /**
   * Cached processed value.
   */
  protected $processed = NULL;

  /**
   * {@inheritdoc}
   */
  public function getValue() {
    if ($this->processed !== NULL) {
      return $this->processed;
    }

    $entity = $this->getParent()->getEntity();

    if (!$entity || !$entity->hasField('field_component_field')) {
      return $this->processed = '';
    }

    $component_field_values = $entity->get('field_component_field');

    if ($component_field_values->isEmpty()) {
      return $this->processed = '';
    }

    $searchable_content = [];
    $discovery = \Drupal::service('component_field.discovery');

    foreach ($component_field_values as $field_item) {
      $component_type = $field_item->get('component_type')->getValue();
      $configuration = $field_item->getConfiguration();

      if (empty($component_type) || empty($configuration)) {
        continue;
      }

      try {
        $content = $this->extractSearchableContent($component_type, $configuration, $discovery);
        if (!empty($content)) {
          $searchable_content[] = $content;
        }
      }
      catch (\Exception $e) {
        \Drupal::logger('component_field')->warning('Error extracting searchable content: @error', [
          '@error' => $e->getMessage(),
        ]);
      }
    }

    return $this->processed = implode(' ', $searchable_content);
  }

  /**
   * Extract searchable content from component configuration.
   */
  protected function extractSearchableContent(string $component_type, array $configuration, $discovery): string {
    $searchable_text = [];

    try {
      foreach ($configuration as $prop_name => $prop_value) {
        if (empty($prop_value)) {
          continue;
        }

        $text = $this->extractTextFromProperty($prop_value);
        if (!empty($text)) {
          $searchable_text[] = $text;
        }
      }
    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error processing component @type: @error', [
        '@type' => $component_type,
        '@error' => $e->getMessage(),
      ]);
    }

    return implode(' ', $searchable_text);
  }

  /**
   * Extract text content from a property value.
   */
  protected function extractTextFromProperty($prop_value): string {
    // Handle text_format fields (rich text)
    if (is_array($prop_value) && isset($prop_value['value'], $prop_value['format'])) {
      return strip_tags($prop_value['value']);
    }

    // Handle processed text.
    if (is_array($prop_value) && isset($prop_value['processed'])) {
      return strip_tags($prop_value['processed']);
    }

    // Handle entity references.
    if (is_array($prop_value) && isset($prop_value['target_id'])) {
      return $this->extractEntityReferenceText($prop_value);
    }

    // Handle simple strings.
    if (is_string($prop_value)) {
      return strip_tags($prop_value);
    }

    // Handle arrays.
    if (is_array($prop_value)) {
      $text_parts = [];
      foreach ($prop_value as $value) {
        if (is_string($value)) {
          $text_parts[] = strip_tags($value);
        }
      }
      return implode(' ', $text_parts);
    }

    return '';
  }

  /**
   * Extract text from entity reference.
   */
  protected function extractEntityReferenceText(array $prop_value): string {
    $entity_type = $prop_value['entity_type'] ?? 'node';

    try {
      $entity = \Drupal::entityTypeManager()->getStorage($entity_type)->load($prop_value['target_id']);

      if (!$entity) {
        return '';
      }

      $text_parts = [];

      // Get entity label.
      if (method_exists($entity, 'label')) {
        $text_parts[] = $entity->label();
      }

      // Extract additional content for nodes.
      if ($entity_type === 'node' && method_exists($entity, 'get')) {
        // Body field.
        if ($entity->hasField('body')) {
          $body = $entity->get('body')->getValue();
          if (!empty($body[0]['value'])) {
            $text_parts[] = strip_tags($body[0]['value']);
          }
        }

        // Other text fields (customize as needed)
        $text_fields = ['field_summary', 'field_description', 'field_excerpt'];
        foreach ($text_fields as $field_name) {
          if ($entity->hasField($field_name)) {
            $field_value = $entity->get($field_name)->getValue();
            if (!empty($field_value[0]['value'])) {
              $text_parts[] = strip_tags($field_value[0]['value']);
            }
          }
        }
      }

      return implode(' ', $text_parts);

    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error extracting entity reference text: @error', [
        '@error' => $e->getMessage(),
      ]);
      return '';
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setValue($value, $notify = TRUE) {
    $this->processed = NULL;
    parent::setValue($value, $notify);
  }

}
