<?php

namespace Drupal\component_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\component_field\Service\ComponentDiscovery;
use Drupal\Component\Utility\NestedArray;
use Drupal\Component\Utility\Html;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Ajax\InvokeCommand;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Enhanced Plugin implementation with proper CKEditor5 AJAX support.
 *
 * @FieldWidget(
 *   id = "component_field_widget",
 *   label = @Translation("Component Field (Enhanced UX)"),
 *   field_types = {
 *     "component_field"
 *   }
 * )
 */
class ComponentFieldWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  use DependencySerializationTrait;

  /**
   * The component discovery service.
   */
  protected ComponentDiscovery $componentDiscovery;

  /**
   * Constructor.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    $field_definition,
    array $settings,
    array $third_party_settings,
    ComponentDiscovery $component_discovery,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->componentDiscovery = $component_discovery;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('component_field.discovery')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    try {
      $item = $items[$delta];
      $component_type = $item->get('component_type')->getValue();

      // SAFETY: Ensure configuration is always an array.
      $configuration = [];
      try {
        $raw_config = $item->getConfiguration();
        $configuration = is_array($raw_config) ? $raw_config : [];
      }
      catch (\Exception $e) {
        \Drupal::logger('component_field')->warning('Error getting configuration for item @delta: @error', [
          '@delta' => $delta,
          '@error' => $e->getMessage(),
        ]);
      }

      // Discover available SDC components with error handling.
      $components = [];
      try {
        $components = $this->componentDiscovery->discoverComponents();
      }
      catch (\Exception $e) {
        \Drupal::logger('component_field')->error('Error discovering components: @error', ['@error' => $e->getMessage()]);
      }

      $field_name = $this->fieldDefinition->getName();
      $wrapper_id = Html::getId("component-field-wrapper-{$field_name}-{$delta}");

      // SAFETY: Ensure element is properly structured.
      $element = array_merge($element, [
        '#type' => 'details',
        '#title' => $this->t('Component Field @delta', ['@delta' => $delta + 1]),
        '#open' => !$this->getSetting('collapsed_by_default'),
        '#attributes' => [
          'class' => ['component-field-item'],
          'id' => $wrapper_id,
          'data-component-delta' => $delta,
        ],
      ]);

      // Component selection with validation.
      $component_options = ['' => $this->t('- Select Component Type -')];
      if (is_array($components) && !empty($components)) {
        foreach ($components as $name => $component) {
          if (is_array($component) && isset($component['label'])) {
            $props_count = isset($component['props']) && is_array($component['props']) ? count($component['props']) : 0;
            $component_options[$name] = $component['label'] . ' (' . $props_count . ' props)';
          }
        }
      }

      $element['component_type'] = [
        '#type' => 'select',
        '#title' => $this->t('Component Type'),
        '#options' => $component_options,
        '#default_value' => $component_type,
        '#required' => TRUE,
        '#ajax' => [
          'callback' => [$this, 'updateConfigFormCallback'],
          'wrapper' => "config-wrapper-{$delta}",
          'method' => 'replace',
          'event' => 'change',
          'disable-refocus' => TRUE,
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Loading component configuration...'),
          ],
        ],
      ];

      // Dynamic configuration form wrapper with proper AJAX handling.
      $element['config_wrapper'] = [
        '#type' => 'container',
        '#attributes' => [
          'id' => "config-wrapper-{$delta}",
          'class' => ['component-field-config-wrapper'],
        ],
        // CRITICAL: Add this to ensure proper CKEditor5 re-initialization.
        '#attached' => [
          'library' => [
            'core/drupal.ajax',
            'editor/drupal.editor',
            'ckeditor5/drupal.ckeditor5',
            'component_field/widget',
          ],
        ],
      ];

      // Get the selected component type (handle AJAX properly)
      $selected_component_type = $this->getSelectedComponentType($form_state, $component_type ?: '', $delta);

      // Build configuration form for selected component.
      if ($selected_component_type && isset($components[$selected_component_type]) && is_array($components[$selected_component_type])) {
        $component_info = $components[$selected_component_type];

        try {
          $config_form = $this->buildDynamicConfigForm(
            $component_info,
            $configuration,
            $delta
          );

          // SAFETY: Ensure config form is a valid render array.
          if (is_array($config_form) && !empty($config_form)) {
            $element['config_wrapper']['config'] = $config_form;
          }
          else {
            $element['config_wrapper']['config'] = [
              '#type' => 'markup',
              '#markup' => '<p><em>' . $this->t('No configuration options available.') . '</em></p>',
            ];
          }
        }
        catch (\Exception $e) {
          \Drupal::logger('component_field')->error('Error building config form for @type: @error', [
            '@type' => $selected_component_type,
            '@error' => $e->getMessage(),
          ]);

          $element['config_wrapper']['config'] = [
            '#type' => 'markup',
            '#markup' => '<p class="error"><em>' . $this->t('Error loading component configuration.') . '</em></p>',
          ];
        }

        // Add component preview if enabled.
        if ($this->getSetting('show_preview')) {
          try {
            $preview_content = $this->generatePreviewHtml($selected_component_type, $configuration);

            $element['config_wrapper']['preview'] = [
              '#type' => 'container',
              '#attributes' => ['class' => ['component-field-preview']],
              'header' => [
                '#type' => 'markup',
                '#markup' => '<div class="component-field-preview-header">
                  <h6 class="component-field-preview-title">👁️ Live Preview</h6>
                  <small>See how your component will appear on the page</small>
                </div>',
              ],
              'content' => [
                '#type' => 'container',
                '#attributes' => [
                  'class' => ['component-field-preview-content'],
                  'id' => "preview-content-{$delta}",
                ],
                '#markup' => $preview_content,
              ],
            ];
          }
          catch (\Exception $e) {
            \Drupal::logger('component_field')->warning('Error generating preview for @type: @error', [
              '@type' => $selected_component_type,
              '@error' => $e->getMessage(),
            ]);
          }
        }

        // Add component info if enabled.
        if ($this->getSetting('show_component_info')) {
          try {
            $element['config_wrapper']['info'] = [
              '#type' => 'details',
              '#title' => $this->t('Component Information'),
              '#open' => FALSE,
              '#attributes' => ['class' => ['component-field-info']],
              'details' => [
                '#type' => 'markup',
                '#markup' => $this->buildComponentInfo($component_info),
              ],
            ];
          }
          catch (\Exception $e) {
            \Drupal::logger('component_field')->warning('Error building component info: @error', ['@error' => $e->getMessage()]);
          }
        }
      }
      else {
        $element['config_wrapper']['placeholder'] = [
          '#type' => 'markup',
          '#markup' => '<div class="component-field-empty-state">' .
          '<div class="icon">🎨</div>' .
          '<p>' . $this->t('Select a component type to configure its properties.') . '</p>' .
          (empty($components) ? '<p><small>' . $this->t('No components found. Check your theme components directory.') . '</small></p>' : '') .
          '</div>',
        ];
      }

      // SAFETY: Hidden field for storing configuration - ensure proper JSON encoding.
      $config_json = '';
      if (!empty($configuration) && is_array($configuration)) {
        try {
          $config_json = json_encode($configuration, JSON_UNESCAPED_UNICODE);
          if (json_last_error() !== JSON_ERROR_NONE) {
            \Drupal::logger('component_field')->warning('JSON encoding error for configuration: @error', [
              '@error' => json_last_error_msg(),
            ]);
            $config_json = '{}';
          }
        }
        catch (\Exception $e) {
          \Drupal::logger('component_field')->warning('Exception during JSON encoding: @error', ['@error' => $e->getMessage()]);
          $config_json = '{}';
        }
      }

      $element['configuration'] = [
        '#type' => 'hidden',
        '#default_value' => $config_json,
      ];

      // Add libraries and settings with CKEditor5 support.
      $element['#attached']['library'][] = 'component_field/widget';
      $element['#attached']['library'][] = 'core/drupal.ajax';
      $element['#attached']['library'][] = 'editor/drupal.editor';
      $element['#attached']['library'][] = 'ckeditor5/drupal.ckeditor5';

      // SAFETY: Only add settings if component exists and is valid.
      if ($selected_component_type && isset($components[$selected_component_type]) && is_array($components[$selected_component_type])) {
        $element['#attached']['drupalSettings']['componentField']['components'][$selected_component_type] = $components[$selected_component_type];
      }

      $element['#element_validate'] = [[$this, 'validateElement']];

      // Add this debugging code right before "return $element;" in formElement()
      try {
        // Check if any element has an invalid 'value' key.
        $this->debugRenderArray($element, "Component Field Element Debug - Delta $delta");
      }
      catch (\Exception $e) {
        \Drupal::logger('component_field')->error('Debug error: @error', ['@error' => $e->getMessage()]);
      }

      return $element;

    }
    catch (\Exception $e) {
      // Catch-all error handling.
      \Drupal::logger('component_field')->error('Critical error in formElement: @error', ['@error' => $e->getMessage()]);

      // Return a minimal safe element.
      return [
        '#type' => 'details',
        '#title' => $this->t('Component Field @delta (Error)', ['@delta' => $delta + 1]),
        '#open' => TRUE,
        'error' => [
          '#type' => 'markup',
          '#markup' => '<div class="messages messages--error">' .
          $this->t('An error occurred loading this component. Please try refreshing the page or contact an administrator.') .
          '</div>',
        ],
        'component_type' => [
          '#type' => 'hidden',
          '#default_value' => '',
        ],
        'configuration' => [
          '#type' => 'hidden',
          '#default_value' => '{}',
        ],
      ];
    }
  }

  /**
   * Debug helper to find invalid render array structure.
   */
  protected function debugRenderArray($array, $context = '', $depth = 0) {
    // Prevent infinite recursion.
    if ($depth > 10) {
      return;
    }

    if (is_array($array)) {
      foreach ($array as $key => $value) {
        if ($key === 'value' && is_string($value)) {
          \Drupal::logger('component_field')->error('FOUND INVALID VALUE KEY: @context - Key: @key, Value: @value', [
            '@context' => $context,
            '@key' => $key,
            '@value' => substr($value, 0, 100),
          ]);
          // Also log the full array structure.
          \Drupal::logger('component_field')->error('FULL ARRAY STRUCTURE: @array', [
            '@array' => print_r($array, TRUE),
          ]);
        }

        if (is_array($value)) {
          $this->debugRenderArray($value, "$context -> $key", $depth + 1);
        }
      }
    }
  }

  /**
   * Build dynamic configuration form with FIXED text_format handling.
   */
  protected function buildDynamicConfigForm(array $component, array $config, int $delta): array {
    // SAFETY: Validate input parameters.
    if (!is_array($component)) {
      return [
        '#type' => 'markup',
        '#markup' => '<p><em>' . $this->t('Invalid component data.') . '</em></p>',
      ];
    }

    $elements = [
      '#type' => 'container',
      '#attributes' => ['class' => ['component-field-config']],
    ];

    // SAFETY: Check if form_config exists and is valid.
    if (empty($component['form_config']) || !is_array($component['form_config'])) {
      $elements['no_props'] = [
        '#type' => 'markup',
        '#markup' => '<p><em>' . $this->t('This component has no configurable properties.') . '</em></p>',
      ];
      return $elements;
    }

    // Build form elements for each prop with individual error handling.
    foreach ($component['form_config'] as $prop_name => $form_element) {
      try {
        // SAFETY: Ensure form_element is an array and prop_name is valid.
        if (!is_array($form_element) || !is_string($prop_name) || empty($prop_name)) {
          continue;
        }

        // Create a safe copy of the form element.
        $element_copy = $form_element;

        // Ensure required properties exist.
        $element_copy['#attributes'] = $element_copy['#attributes'] ?? [];
        $element_copy['#attributes']['class'] = $element_copy['#attributes']['class'] ?? [];
        $element_copy['#attributes']['class'][] = 'component-field-config-field';
        $element_copy['#attributes']['data-prop-name'] = $prop_name;

        // Apply current configuration values safely with proper type conversion.
        if (isset($config[$prop_name])) {
          $config_value = $config[$prop_name];
          $element_type = $form_element['#type'] ?? 'textfield';

          // Handle different element types appropriately.
          if ($element_type === 'text_format') {
            // CRITICAL FIX: Proper text_format field handling.
            // Ensure we have a unique field ID to prevent CKEditor conflicts.
            $element_copy['#id'] = Html::getId("text-format-{$prop_name}-{$delta}");

            // Set proper wrapper attributes for AJAX.
            $element_copy['#prefix'] = '<div id="text-format-wrapper-' . $prop_name . '-' . $delta . '" class="text-format-wrapper-ajax">';
            $element_copy['#suffix'] = '</div>';

            if (is_array($config_value)) {
              if (isset($config_value['value'], $config_value['format'])) {
                // Standard text_format structure - THIS ONE IS CORRECT NOW.
                $element_copy['#default_value'] = (string) $config_value['value'];
                $element_copy['#format'] = (string) $config_value['format'];
              }
              elseif (isset($config_value['processed'])) {
                // Structure with processed content - FIX THIS.
                $element_copy['#default_value'] = (string) ($config_value['value'] ?? $config_value['processed']);
                $element_copy['#format'] = (string) ($config_value['format'] ?? 'basic_html');
              }
              else {
                // Array but not proper text_format structure - FIX THIS.
                $element_copy['#default_value'] = is_string($config_value) ? $config_value : json_encode($config_value);
                $element_copy['#format'] = 'basic_html';
              }
            }
            else {
              // Simple string value - FIX THIS.
              $element_copy['#default_value'] = (string) $config_value;
              $element_copy['#format'] = 'basic_html';
            }

            // CRITICAL: Ensure format is properly set.
            if (!isset($element_copy['#format'])) {
              $element_copy['#format'] = $element_copy['#default_value']['format'] ?? 'basic_html';
            }

            // CRITICAL: Attach required libraries for CKEditor5.
            $element_copy['#attached']['library'][] = 'core/drupal.ajax';
            $element_copy['#attached']['library'][] = 'editor/drupal.editor';
            $element_copy['#attached']['library'][] = 'ckeditor5/drupal.ckeditor5';

            // Add special AJAX handling for text_format.
            $element_copy['#ajax'] = [
              'disable-refocus' => TRUE,
              'effect' => 'none',
              'event' => 'change',
            ];

          }
          elseif ($element_type === 'textarea') {
            // Handle textarea fields - convert arrays to JSON strings.
            if (is_array($config_value)) {
              try {
                $element_copy['#default_value'] = json_encode($config_value, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
              }
              catch (\Exception $e) {
                \Drupal::logger('component_field')->warning('Error JSON encoding array value for textarea @prop: @error', [
                  '@prop' => $prop_name,
                  '@error' => $e->getMessage(),
                ]);
                $element_copy['#default_value'] = '';
              }
            }
            else {
              $element_copy['#default_value'] = (string) $config_value;
            }
          }
          elseif ($element_type === 'checkbox') {
            // Handle checkbox - ensure boolean.
            $element_copy['#default_value'] = (bool) $config_value;
          }
          elseif (in_array($element_type, ['number', 'range'])) {
            // Handle numeric fields.
            $element_copy['#default_value'] = is_numeric($config_value) ? $config_value : 0;
          }
          elseif ($element_type === 'select' || $element_type === 'radios') {
            // Handle select/radio - ensure valid option.
            $options = $form_element['#options'] ?? [];
            if (isset($options[$config_value])) {
              $element_copy['#default_value'] = $config_value;
            }
            else {
              // Use first option as fallback.
              $element_copy['#default_value'] = array_keys($options)[0] ?? '';
            }
          }
          else {
            // For all other field types, convert to string unless it's already appropriate.
            if (is_array($config_value)) {
              // Convert arrays to JSON for non-textarea, non-text_format fields.
              try {
                $element_copy['#default_value'] = json_encode($config_value);
              }
              catch (\Exception $e) {
                $element_copy['#default_value'] = '';
              }
            }
            else {
              $element_copy['#default_value'] = $config_value;
            }
          }
        }

        // Add AJAX for live preview updates (non-text_format fields)
        if ($this->getSetting('show_preview') && ($form_element['#type'] ?? '') !== 'text_format') {
          $element_copy['#ajax'] = [
            'callback' => [$this, 'updatePreviewCallback'],
            'wrapper' => "preview-content-{$delta}",
            'event' => 'change',
            'disable-refocus' => TRUE,
            'progress' => ['type' => 'none'],
          ];
        }

        $elements[$prop_name] = $element_copy;

      }
      catch (\Exception $e) {
        \Drupal::logger('component_field')->warning('Error building form element for prop @prop: @error', [
          '@prop' => $prop_name,
          '@error' => $e->getMessage(),
        ]);

        // Add a fallback element.
        $elements[$prop_name] = [
          '#type' => 'markup',
          '#markup' => '<p class="error"><em>' . $this->t('Error loading @prop configuration.', ['@prop' => $prop_name]) . '</em></p>',
        ];
      }
    }

    return $elements;
  }

  /**
   * AJAX callback for component type change with ENHANCED CKEditor5 handling.
   */
  public function updateConfigFormCallback(array &$form, FormStateInterface $form_state) {
    try {
      $trigger = $form_state->getTriggeringElement();

      if (!$trigger || !isset($trigger['#parents'])) {
        return [
          '#type' => 'container',
          '#markup' => '<p class="error">' . $this->t('Error: Unable to load component configuration.') . '</p>',
        ];
      }

      // Navigate to the config_wrapper element.
      $parents = array_slice($trigger['#parents'], 0, -1);
      $parents[] = 'config_wrapper';

      $config_element = NestedArray::getValue($form, $parents);

      if ($config_element === NULL || !is_array($config_element)) {
        return [
          '#type' => 'container',
          '#markup' => '<p class="error">' . $this->t('Error: Component configuration not found.') . '</p>',
        ];
      }

      // CRITICAL FIX: Create proper AJAX response to reinitialize CKEditor5.
      $response = new AjaxResponse();
      $response->addCommand(new ReplaceCommand('#' . $config_element['#attributes']['id'], $config_element));

      // CRITICAL: Re-attach CKEditor5 behaviors after AJAX replace.
      $response->addCommand(new InvokeCommand(NULL, 'trigger', ['ckeditorReinitialized']));

      return $response;

    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->error('Error in updateConfigFormCallback: @error', ['@error' => $e->getMessage()]);

      return [
        '#type' => 'container',
        '#markup' => '<p class="error">' . $this->t('Error loading component configuration. Please try again.') . '</p>',
      ];
    }
  }

  /**
   * Get the selected component type, handling AJAX properly with validation.
   */
  protected function getSelectedComponentType(FormStateInterface $form_state, string $default_type, int $delta): string {
    try {
      $trigger_element = $form_state->getTriggeringElement();

      if ($trigger_element && isset($trigger_element['#parents']) && is_array($trigger_element['#parents'])) {
        $parents = $trigger_element['#parents'];

        // Check if this is our component type selector.
        if (end($parents) === 'component_type') {
          $user_input = $form_state->getUserInput();
          $field_parents = array_slice($parents, 0, -1);
          $field_values = NestedArray::getValue($user_input, $field_parents);

          if (isset($field_values['component_type']) && is_string($field_values['component_type'])) {
            return $field_values['component_type'];
          }
        }
      }
    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error getting selected component type: @error', ['@error' => $e->getMessage()]);
    }

    return is_string($default_type) ? $default_type : '';
  }

  /**
   * Generate preview HTML with comprehensive error handling.
   */
  protected function generatePreviewHtml(string $component_type, array $config): string {
    try {
      if (empty($component_type)) {
        return '<div class="component-field-preview-empty">
          <span class="component-field-preview-icon">🎨</span>
          <p>Select a component to see preview</p>
        </div>';
      }

      $components = $this->componentDiscovery->discoverComponents();
      if (!isset($components[$component_type]) || !is_array($components[$component_type])) {
        return '<div class="component-field-preview-error">
          <span class="component-field-preview-icon">❌</span>
          <p>Component "' . htmlspecialchars($component_type) . '" not found</p>
        </div>';
      }

      // Generate proper component HTML preview.
      return $this->generateComponentPreview($component_type, $config, $components[$component_type]);

    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error generating preview HTML: @error', ['@error' => $e->getMessage()]);

      return '<div class="component-field-preview-error">
        <span class="component-field-preview-icon">⚠️</span>
        <p>Preview temporarily unavailable</p>
      </div>';
    }
  }

  /**
   * Generate actual component preview HTML with error handling.
   */
  protected function generateComponentPreview(string $component_type, array $config, array $component_info): string {
    try {
      // Apply defaults from component schema.
      $final_config = $this->applyDefaults($config, $component_info);

      switch ($component_type) {
        case 'test':
          return $this->generateTestPreview($final_config);

        case 'button':
          return $this->generateButtonPreview($final_config);

        case 'card':
          return $this->generateCardPreview($final_config);

        case 'typography':
          return $this->generateTypographyPreview($final_config);

        default:
          return $this->generateGenericPreview($component_type, $final_config, $component_info);
      }
    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error generating component preview for @type: @error', [
        '@type' => $component_type,
        '@error' => $e->getMessage(),
      ]);

      return '<div class="component-field-preview-error">
        <span class="component-field-preview-icon">⚠️</span>
        <p>Preview error for ' . htmlspecialchars($component_type) . '</p>
      </div>';
    }
  }

  /**
   * Apply default values from component schema with validation.
   */
  protected function applyDefaults(array $config, array $component_info): array {
    if (!is_array($config)) {
      $config = [];
    }

    $final_config = $config;

    try {
      if (!empty($component_info['props']) && is_array($component_info['props'])) {
        foreach ($component_info['props'] as $prop_name => $prop_schema) {
          if (is_array($prop_schema) && !isset($final_config[$prop_name]) && isset($prop_schema['default'])) {
            $final_config[$prop_name] = $prop_schema['default'];
          }
        }
      }
    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error applying defaults: @error', ['@error' => $e->getMessage()]);
    }

    return $final_config;
  }

  /**
   * Generate Test component preview with validation.
   */
  protected function generateTestPreview(array $config): string {
    $variant = isset($config['variant']) && is_string($config['variant']) ? $config['variant'] : 'primary';
    $size = isset($config['size']) && is_string($config['size']) ? $config['size'] : 'medium';
    $disabled = !empty($config['disabled']);
    $title = isset($config['title']) && is_string($config['title']) ? $config['title'] : 'Test Component';
    $content = isset($config['content']) && is_string($config['content']) ? $config['content'] : 'This is a test of the Component Field system.';
    $counter = isset($config['counter']) && is_numeric($config['counter']) ? (int) $config['counter'] : 0;

    $classes = [
      'ComponentTest-root',
      'ComponentTest-' . htmlspecialchars($variant),
      'ComponentTest-size' . ucfirst(htmlspecialchars($size)),
    ];

    if ($disabled) {
      $classes[] = 'ComponentTest-disabled';
    }

    $html = '<div class="component-field-preview-component">';
    $html .= '<div class="' . implode(' ', $classes) . '">';

    // Header.
    $html .= '<div class="ComponentTest-header">';
    $html .= '<h4 class="ComponentTest-title">' . htmlspecialchars($title) . '</h4>';
    $html .= '<div class="ComponentTest-meta">';
    $html .= '<span class="ComponentTest-variant">' . htmlspecialchars($variant) . '</span>';
    $html .= '<span class="ComponentTest-size">' . htmlspecialchars($size) . '</span>';
    $html .= '</div></div>';

    // Content.
    $html .= '<div class="ComponentTest-content">';
    $html .= '<div class="ComponentTest-description">' . htmlspecialchars($content) . '</div>';
    $html .= '<div class="ComponentTest-counter"><strong>Counter:</strong> ' . $counter . '</div>';
    $html .= '<div class="ComponentTest-status">';
    $html .= '<span>Status:</span>';
    $html .= '<span class="ComponentTest-statusBadge ' . ($disabled ? 'disabled' : 'enabled') . '">';
    $html .= $disabled ? 'Disabled' : 'Enabled';
    $html .= '</span></div></div>';

    // Footer.
    $html .= '<div class="ComponentTest-footer">✅ Component Field Test Working!</div>';
    $html .= '</div></div>';

    return $html;
  }

  /**
   * Generate Button preview with validation.
   */
  protected function generateButtonPreview(array $config): string {
    $variant = isset($config['variant']) && is_string($config['variant']) ? $config['variant'] : 'contained';
    $color = isset($config['color']) && is_string($config['color']) ? $config['color'] : 'primary';
    $size = isset($config['size']) && is_string($config['size']) ? $config['size'] : 'medium';
    $disabled = !empty($config['disabled']);

    // Handle label - could be string or rich text array.
    $label = 'Button';
    if (isset($config['label'])) {
      if (is_array($config['label']) && isset($config['label']['processed'])) {
        $label = $config['label']['processed'];
      }
      elseif (is_array($config['label']) && isset($config['label']['value'])) {
        $label = $config['label']['value'];
      }
      elseif (is_string($config['label'])) {
        $label = $config['label'];
      }
    }

    $classes = [
      'ComponentButton-root',
      'ComponentButtonBase-root',
      'ComponentButton-' . htmlspecialchars($variant),
      'ComponentButton-' . htmlspecialchars($variant) . ucfirst(htmlspecialchars($color)),
      'ComponentButton-size' . ucfirst(htmlspecialchars($size)),
    ];

    if ($disabled) {
      $classes[] = 'Component-disabled';
    }

    $html = '<div class="component-field-preview-component">';
    $html .= '<button class="' . implode(' ', $classes) . '"';
    if ($disabled) {
      $html .= ' disabled';
    }
    $html .= '>';
    $html .= '<span class="ComponentButton-label">' . $label . '</span>';
    $html .= '</button></div>';

    return $html;
  }

  /**
   * Generate Card preview with validation.
   */
  protected function generateCardPreview(array $config): string {
    $elevation = isset($config['elevation']) && is_numeric($config['elevation']) ? (int) $config['elevation'] : 1;
    $variant = isset($config['variant']) && is_string($config['variant']) ? $config['variant'] : 'elevation';
    $title = isset($config['title']) && is_string($config['title']) ? $config['title'] : '';

    // Handle content - could be string or rich text array.
    $content = 'This is sample card content to show how your card will look.';
    if (isset($config['content'])) {
      if (is_array($config['content']) && isset($config['content']['processed'])) {
        $content = $config['content']['processed'];
      }
      elseif (is_array($config['content']) && isset($config['content']['value'])) {
        $content = $config['content']['value'];
      }
      elseif (is_string($config['content'])) {
        $content = $config['content'];
      }
    }

    $classes = [
      'ComponentCard-root',
      'ComponentPaper-root',
      $variant === 'outlined' ? 'ComponentPaper-outlined' : 'ComponentPaper-elevation' . $elevation,
    ];

    $html = '<div class="component-field-preview-component">';
    $html .= '<div class="' . implode(' ', $classes) . '" style="max-width: 300px;">';

    if (!empty($title)) {
      $html .= '<div class="ComponentCardHeader-root">';
      $html .= '<h6 class="ComponentCardHeader-title">' . htmlspecialchars($title) . '</h6>';
      $html .= '</div>';
    }

    $html .= '<div class="ComponentCardContent-root">';
    $html .= '<div>' . $content . '</div>';
    $html .= '</div>';
    $html .= '</div></div>';

    return $html;
  }

  /**
   * Generate Typography preview with validation.
   */
  protected function generateTypographyPreview(array $config): string {
    $variant = isset($config['variant']) && is_string($config['variant']) ? $config['variant'] : 'body1';

    // Handle content - could be string or rich text array.
    $text = 'Sample typography text';
    if (isset($config['content'])) {
      if (is_array($config['content']) && isset($config['content']['processed'])) {
        $text = $config['content']['processed'];
      }
      elseif (is_array($config['content']) && isset($config['content']['value'])) {
        $text = $config['content']['value'];
      }
      elseif (is_string($config['content'])) {
        $text = $config['content'];
      }
    }

    $classes = ['ComponentTypography-root', 'ComponentTypography-' . htmlspecialchars($variant)];
    $component = $this->getDefaultComponent($variant);

    $html = '<div class="component-field-preview-component">';
    $html .= '<' . htmlspecialchars($component) . ' class="' . implode(' ', $classes) . '">';
    $html .= $text;
    $html .= '</' . htmlspecialchars($component) . '>';
    $html .= '</div>';

    return $html;
  }

  /**
   * Generate generic preview for unknown components with validation.
   */
  protected function generateGenericPreview(string $component_type, array $config, array $component_info): string {
    $label = isset($component_info['label']) && is_string($component_info['label']) ? $component_info['label'] : $component_type;

    $html = '<div class="component-field-preview-component component-field-preview-generic">';
    $html .= '<div class="component-field-preview-header">';
    $html .= '<strong>📦 ' . htmlspecialchars($label) . '</strong>';
    $html .= '</div>';

    if (!empty($config) && is_array($config)) {
      $html .= '<div class="component-field-preview-sample">';
      $html .= '<p>Component configured with:</p>';
      $html .= '<ul>';

      $count = 0;
      foreach ($config as $key => $value) {
        if ($count >= 3) {
          break;
        }

        // Handle different value types for display.
        if (is_array($value)) {
          if (isset($value['processed'])) {
            $display_value = 'Rich Text';
          }
          elseif (isset($value['value'])) {
            $display_value = 'Text: ' . substr((string) $value['value'], 0, 20) . '...';
          }
          else {
            $display_value = 'Array (' . count($value) . ' items)';
          }
        }
        else {
          $display_value = (string) $value;
          if (strlen($display_value) > 30) {
            $display_value = substr($display_value, 0, 27) . '...';
          }
        }

        $html .= '<li><code>' . htmlspecialchars($key) . '</code>: ' . htmlspecialchars($display_value) . '</li>';
        $count++;
      }

      if (count($config) > 3) {
        $html .= '<li><em>... and ' . (count($config) - 3) . ' more</em></li>';
      }
      $html .= '</ul></div>';
    }
    else {
      $html .= '<div class="component-field-preview-empty">No configuration set</div>';
    }

    $html .= '</div>';
    return $html;
  }

  /**
   * Get default HTML component for typography variants.
   */
  protected function getDefaultComponent(string $variant): string {
    $mapping = [
      'h1' => 'h1',
      'h2' => 'h2',
      'h3' => 'h3',
      'h4' => 'h4',
      'h5' => 'h5',
      'h6' => 'h6',
      'subtitle1' => 'h6',
      'subtitle2' => 'h6',
      'body1' => 'p',
      'body2' => 'p',
      'caption' => 'span',
      'button' => 'span',
      'overline' => 'span',
    ];
    return $mapping[$variant] ?? 'p';
  }

  /**
   * Build component information display with validation.
   */
  protected function buildComponentInfo(array $component_info): string {
    try {
      $html = '<div class="component-field-info-content">';

      if (!empty($component_info['description']) && is_string($component_info['description'])) {
        $html .= '<p><strong>Description:</strong> ' . htmlspecialchars($component_info['description']) . '</p>';
      }

      if (!empty($component_info['relative_path']) && is_string($component_info['relative_path'])) {
        $html .= '<p><strong>Source:</strong> <code>' . htmlspecialchars($component_info['relative_path']) . '</code></p>';
      }

      $props_count = isset($component_info['props']) && is_array($component_info['props']) ? count($component_info['props']) : 0;
      $html .= '<p><strong>Properties:</strong> ' . $props_count . '</p>';

      if ($props_count > 0) {
        $html .= '<details><summary>View Properties</summary><ul>';
        foreach ($component_info['props'] as $prop_name => $prop_info) {
          if (!is_string($prop_name) || !is_array($prop_info)) {
            continue;
          }

          $html .= '<li><code>' . htmlspecialchars($prop_name) . '</code>';
          if (isset($prop_info['type']) && is_string($prop_info['type'])) {
            $html .= ' (' . htmlspecialchars($prop_info['type']) . ')';
          }
          if (isset($prop_info['description']) && is_string($prop_info['description'])) {
            $html .= ' - ' . htmlspecialchars($prop_info['description']);
          }
          $html .= '</li>';
        }
        $html .= '</ul></details>';
      }

      $html .= '</div>';
      return $html;

    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error building component info: @error', ['@error' => $e->getMessage()]);
      return '<p><em>Component information unavailable</em></p>';
    }
  }

  /**
   * Enhanced validate and store configuration with comprehensive error handling.
   */
  public function validateElement(array $element, FormStateInterface $form_state) {
    try {
      $values = $form_state->getValue($element['#parents']);

      if (!is_array($values)) {
        return;
      }

      if (!empty($values['component_type']) && is_string($values['component_type'])) {
        $config = [];

        // Extract configuration from form.
        if (isset($values['config_wrapper']['config']) && is_array($values['config_wrapper']['config'])) {
          foreach ($values['config_wrapper']['config'] as $key => $value) {
            if (!is_string($key)) {
              continue;
            }

            try {
              $processed_value = $this->processConfigValue($key, $value);

              // Only include non-empty values but allow 0 and false.
              if ($processed_value !== '' && $processed_value !== NULL) {
                $config[$key] = $processed_value;
              }
            }
            catch (\Exception $e) {
              \Drupal::logger('component_field')->warning('Error processing config value for @key: @error', [
                '@key' => $key,
                '@error' => $e->getMessage(),
              ]);
            }
          }
        }

        // Store as JSON with error handling.
        try {
          $json_config = json_encode($config, JSON_UNESCAPED_UNICODE);
          if (json_last_error() === JSON_ERROR_NONE) {
            $form_state->setValueForElement($element['configuration'], $json_config);
          }
          else {
            \Drupal::logger('component_field')->error('JSON encoding failed: @error', ['@error' => json_last_error_msg()]);
            $form_state->setValueForElement($element['configuration'], '{}');
          }
        }
        catch (\Exception $e) {
          \Drupal::logger('component_field')->error('Configuration encoding error: @error', ['@error' => $e->getMessage()]);
          $form_state->setValueForElement($element['configuration'], '{}');
        }
      }
      else {
        // Clear configuration if no component type selected.
        $form_state->setValueForElement($element['configuration'], '{}');
      }

    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->error('Error in validateElement: @error', ['@error' => $e->getMessage()]);
      // Set a safe default.
      $form_state->setValueForElement($element['configuration'], '{}');
    }
  }

  /**
   * Process configuration value with error handling.
   */
  protected function processConfigValue(string $key, $value) {
    try {
      // Handle text format fields (rich text)
      if (is_array($value) && isset($value['value'], $value['format'])) {
        return [
          'value' => (string) $value['value'],
          'format' => (string) $value['format'],
          'processed' => check_markup((string) $value['value'], (string) $value['format']),
        ];
      }

      // Handle entity references.
      if (is_array($value) && isset($value['target_id'])) {
        return [
          'target_id' => $value['target_id'],
          'entity_type' => $value['entity_type'] ?? 'node',
        ];
      }

      // Handle media library.
      if (is_array($value) && isset($value['selection'])) {
        return is_array($value['selection']) ? array_values($value['selection']) : [];
      }

      // Handle JSON arrays/objects from textarea.
      if (is_string($value) && (strpos($value, '[') === 0 || strpos($value, '{') === 0)) {
        $decoded = json_decode($value, TRUE);
        return $decoded ?? $value;
      }

      return $value;

    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error processing config value for @key: @error', [
        '@key' => $key,
        '@error' => $e->getMessage(),
      ]);
      return $value;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'show_preview' => TRUE,
      'collapsed_by_default' => FALSE,
      'show_component_info' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    $element['show_preview'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show component preview'),
      '#default_value' => $this->getSetting('show_preview'),
      '#description' => $this->t('Show a live preview of the component while editing.'),
    ];

    $element['collapsed_by_default'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Collapse components by default'),
      '#default_value' => $this->getSetting('collapsed_by_default'),
      '#description' => $this->t('Show components in collapsed state initially.'),
    ];

    $element['show_component_info'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show component information'),
      '#default_value' => $this->getSetting('show_component_info'),
      '#description' => $this->t('Display additional information about components.'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    if ($this->getSetting('show_preview')) {
      $summary[] = $this->t('Preview: Enabled');
    }

    if ($this->getSetting('collapsed_by_default')) {
      $summary[] = $this->t('Default state: Collapsed');
    }

    if ($this->getSetting('show_component_info')) {
      $summary[] = $this->t('Component info: Shown');
    }

    try {
      $components = $this->componentDiscovery->discoverComponents();
      $summary[] = $this->t('Available components: @count', [
        '@count' => count($components),
      ]);
    }
    catch (\Exception $e) {
      $summary[] = $this->t('Components: Error loading');
    }

    return $summary;
  }

}
