<?php

namespace Drupal\component_field\Service;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Symfony\Component\Yaml\Yaml;

/**
 * Enhanced service for discovering SDC components with FIXED text format handling.
 */
class ComponentDiscovery {

  use DependencySerializationTrait;

  const CACHE_ID = 'component_field_discovered';

  protected CacheBackendInterface $cache;
  protected ThemeHandlerInterface $themeHandler;
  protected FileSystemInterface $fileSystem;
  protected LoggerChannelFactoryInterface $loggerFactory;

  public function __construct(
    CacheBackendInterface $cache,
    ThemeHandlerInterface $theme_handler,
    FileSystemInterface $file_system,
    LoggerChannelFactoryInterface $logger_factory,
  ) {
    $this->cache = $cache;
    $this->themeHandler = $theme_handler;
    $this->fileSystem = $file_system;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * ENHANCED: Build text element with FIXED format support and validation.
   */
  protected function buildTextElement(array $element, array $prop_schema, ?string $format): array {
    try {
      switch ($format) {
        case 'html':
          // CRITICAL FIX: Properly configure text_format field.
          $element['#type'] = 'text_format';

          // Get available text formats safely with fallbacks.
          $formats = filter_formats();
          $allowed_formats = [];

          // Prefer formats that support rich text editing.
          $preferred_formats = ['full_html', 'basic_html', 'restricted_html'];
          foreach ($preferred_formats as $format_id) {
            if (isset($formats[$format_id])) {
              $allowed_formats[] = $format_id;
            }
          }

          // Fallback to any available format.
          if (empty($allowed_formats) && !empty($formats)) {
            $allowed_formats = array_keys($formats);
          }

          // Final fallback.
          if (empty($allowed_formats)) {
            $allowed_formats = ['plain_text'];
          }

          // CRITICAL: Proper text_format configuration.
          $element['#allowed_formats'] = $allowed_formats;
          $element['#format'] = reset($allowed_formats);
          $element['#title'] .= ' (Rich Text)';
          $element['#rows'] = 6;

          $default_text = '';
          if (isset($prop_schema['default'])) {
            if (is_string($prop_schema['default'])) {
              $default_text = $prop_schema['default'];
            }
            elseif (is_array($prop_schema['default']) && isset($prop_schema['default']['value'])) {
              $default_text = $prop_schema['default']['value'];
            }
          }

          // For text_format, set both the value and format properly.
          $element['#default_value'] = $default_text;
          $element['#format'] = reset($allowed_formats);

          if (!isset($element['#attributes'])) {
            $element['#attributes'] = [];
          }
          $element['#attributes']['data-text-format'] = TRUE;
          $element['#attributes']['data-rich-text'] = TRUE;

          // CRITICAL: Attach required libraries.
          if (!isset($element['#attached'])) {
            $element['#attached'] = [];
          }
          if (!isset($element['#attached']['library'])) {
            $element['#attached']['library'] = [];
          }

          $element['#attached']['library'] = array_merge($element['#attached']['library'], [
            'core/drupal.ajax',
            'editor/drupal.editor',
            'ckeditor5/drupal.ckeditor5',
            'filter/drupal.filter',
          ]);

          // CRITICAL: Add settings for text format.
          if (!isset($element['#attached']['drupalSettings'])) {
            $element['#attached']['drupalSettings'] = [];
          }

          $element['#attached']['drupalSettings']['filter'] = [
            'formats' => array_fill_keys($allowed_formats, [
              'editor' => 'ckeditor5',
              'editorSettings' => [],
            ]),
          ];

          break;

        case 'raw_html':
          $element['#type'] = 'textarea';
          $element['#rows'] = 6;
          $element['#title'] .= ' (Raw HTML)';
          $element['#description'] .= $element['#description'] ? '<br>' : '';
          $element['#description'] .= '<strong>Warning:</strong> Raw HTML - use with caution.';

          // Add data attribute for template handling.
          if (!isset($element['#attributes'])) {
            $element['#attributes'] = [];
          }
          $element['#attributes']['data-rich-text'] = TRUE;
          break;

        case 'markdown':
          $element['#type'] = 'textarea';
          $element['#rows'] = 8;
          $element['#title'] .= ' (Markdown)';
          $element['#description'] .= $element['#description'] ? '<br>' : '';
          $element['#description'] .= 'Supports Markdown formatting.';
          break;

        default:
          $element['#type'] = 'textarea';
          $element['#rows'] = 4;
          break;
      }

      // CRITICAL: Add proper AJAX handling for text_format fields.
      if ($element['#type'] === 'text_format') {
        $element['#ajax'] = [
          'disable-refocus' => TRUE,
          'effect' => 'none',
          'event' => 'change',
          'progress' => ['type' => 'none'],
        ];
      }

      return $element;

    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error building text element: @error', ['@error' => $e->getMessage()]);

      // Fallback to simple textarea with rich text support.
      $element['#type'] = 'textarea';
      $element['#rows'] = 4;
      $element['#attributes']['data-rich-text'] = TRUE;
      return $element;
    }
  }

  /**
   * ENHANCED: Convert YAML prop schema to Drupal form element with FIXED text_format handling.
   */
  protected function yamlPropToFormElement(string $name, array $prop_schema): ?array {
    try {
      // SAFETY: Validate inputs.
      if (!is_string($name) || empty($name) || !is_array($prop_schema)) {
        return NULL;
      }

      $element = [
        '#title' => $this->getStringValue($prop_schema, 'title', $this->humanizePropName($name)),
        '#description' => $this->getStringValue($prop_schema, 'description', ''),
        '#attributes' => ['class' => ['component-field-config-field']],
        '#required' => FALSE,
      ];

      // Set default value safely.
      if (isset($prop_schema['default'])) {
        $element['#default_value'] = $prop_schema['default'];
      }

      // Determine form element type based on YAML schema.
      $type = $this->getStringValue($prop_schema, 'type', 'string');
      $format = $this->getStringValue($prop_schema, 'format');

      switch ($type) {
        case 'text':
          return $this->buildTextElement($element, $prop_schema, $format);

        case 'entity_reference':
          return $this->buildEntityReferenceElement($element, $prop_schema);

        case 'string':
          if (!empty($prop_schema['enum']) && is_array($prop_schema['enum'])) {
            $element['#type'] = 'select';
            $element['#options'] = $this->buildSelectOptions($prop_schema['enum']);
            $element['#empty_option'] = '- Select -';
          }
          else {
            $element['#type'] = 'textfield';
            if (isset($prop_schema['maxLength']) && is_numeric($prop_schema['maxLength'])) {
              $element['#maxlength'] = (int) $prop_schema['maxLength'];
            }
          }
          break;

        case 'integer':
        case 'number':
          if (!empty($prop_schema['enum']) && is_array($prop_schema['enum'])) {
            $element['#type'] = 'select';
            $element['#options'] = $this->buildSelectOptions($prop_schema['enum']);
            $element['#empty_option'] = '- Select -';
          }
          else {
            $element['#type'] = 'number';
            if (isset($prop_schema['minimum']) && is_numeric($prop_schema['minimum'])) {
              $element['#min'] = $prop_schema['minimum'];
            }
            if (isset($prop_schema['maximum']) && is_numeric($prop_schema['maximum'])) {
              $element['#max'] = $prop_schema['maximum'];
            }
          }
          break;

        case 'boolean':
          $element['#type'] = 'checkbox';
          $element['#default_value'] = isset($prop_schema['default']) ? (bool) $prop_schema['default'] : FALSE;
          break;

        case 'array':
          return $this->buildArrayElement($element, $prop_schema);

        case 'object':
          $element['#type'] = 'textarea';
          $element['#rows'] = 4;
          $element['#title'] .= ' (JSON Object)';
          $element['#description'] .= $element['#description'] ? '<br>' : '';
          $element['#description'] .= 'Enter as JSON object, e.g., {"key": "value"}';
          $element['#element_validate'] = [[$this, 'validateJsonObject']];
          break;

        default:
          $element['#type'] = 'textfield';
      }

      return $element;

    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error converting prop @name to form element: @error', [
        '@name' => $name,
        '@error' => $e->getMessage(),
      ]);
      return NULL;
    }
  }

  /**
   * Discover all SDC components with comprehensive error handling.
   */
  public function discoverComponents(bool $reset_cache = FALSE): array {
    $logger = $this->loggerFactory->get('component_field');

    if (!$reset_cache && $cached = $this->cache->get(self::CACHE_ID)) {
      // SAFETY: Validate cached data.
      if (is_array($cached->data)) {
        $logger->info('Returning cached SDC components: @count', ['@count' => count($cached->data)]);
        return $cached->data;
      }
      else {
        $logger->warning('Cached data is not an array, refreshing cache');
      }
    }

    $logger->info('Starting SDC component discovery...');

    $components = [];

    try {
      $default_theme = $this->themeHandler->getDefault();
      $logger->info('Default theme: @theme', ['@theme' => $default_theme]);

      if (empty($default_theme)) {
        $logger->error('No default theme found');
        return [];
      }

      $theme_path = $this->themeHandler->getTheme($default_theme)->getPath();
      $logger->info('Theme path: @path', ['@path' => $theme_path]);

      if (empty($theme_path)) {
        $logger->error('Theme path is empty');
        return [];
      }

    }
    catch (\Exception $e) {
      $logger->error('Failed to get theme information: @error', ['@error' => $e->getMessage()]);
      return [];
    }

    // Define possible component directories for SDC.
    $component_dirs = [
    // Standard SDC location.
      'components',
    // Alternative location.
      'src/components',
    ];

    foreach ($component_dirs as $dir) {
      try {
        $full_path = DRUPAL_ROOT . '/' . $theme_path . '/' . $dir;
        $logger->info('Checking SDC directory: @path', ['@path' => $full_path]);

        if (is_dir($full_path) && is_readable($full_path)) {
          $logger->info('Directory exists and is readable, scanning for SDC components...');

          $discovered = $this->scanForSdcComponents($full_path);

          // SAFETY: Ensure discovered is an array.
          if (is_array($discovered)) {
            $logger->info('Found @count SDC components in @dir', [
              '@count' => count($discovered),
              '@dir' => $dir,
            ]);
            $components = array_merge($components, $discovered);
          }
          else {
            $logger->warning('scanForSdcComponents did not return an array for @dir', ['@dir' => $dir]);
          }
        }
        else {
          $logger->info('SDC directory does not exist or is not readable: @path', ['@path' => $full_path]);
        }
      }
      catch (\Exception $e) {
        $logger->error('Error processing SDC directory @dir: @error', [
          '@dir' => $dir,
          '@error' => $e->getMessage(),
        ]);
      }
    }

    $logger->info('Total SDC components found: @count', ['@count' => count($components)]);

    // Cache the results with validation.
    try {
      $this->cache->set(self::CACHE_ID, $components, CacheBackendInterface::CACHE_PERMANENT, ['component_field']);
      $logger->info('Cached @count SDC components', ['@count' => count($components)]);
    }
    catch (\Exception $e) {
      $logger->error('Error caching components: @error', ['@error' => $e->getMessage()]);
    }

    return $components;
  }

  /**
   * Scan directory for SDC component directories with robust error handling.
   */
  protected function scanForSdcComponents(string $directory): array {
    $logger = $this->loggerFactory->get('component_field');
    $components = [];

    try {
      if (!is_dir($directory) || !is_readable($directory)) {
        $logger->warning('Directory is not readable: @dir', ['@dir' => $directory]);
        return [];
      }

      $iterator = new \DirectoryIterator($directory);

      foreach ($iterator as $item) {
        try {
          if ($item->isDot() || !$item->isDir() || !$item->isReadable()) {
            continue;
          }

          $component_dir = $item->getPathname();
          $component_name = $item->getBasename();

          // SAFETY: Validate component name.
          if (empty($component_name) || !is_string($component_name)) {
            continue;
          }

          $logger->debug('Checking potential SDC directory: @dir', ['@dir' => $component_name]);

          // Look for .component.yml file.
          $yml_file = $component_dir . '/' . $component_name . '.component.yml';

          if (file_exists($yml_file) && is_readable($yml_file)) {
            $logger->info('Found SDC component: @name', ['@name' => $component_name]);

            $component_data = $this->parseSdcComponent($component_dir, $component_name, $yml_file);

            // SAFETY: Validate component data.
            if (is_array($component_data) && !empty($component_data)) {
              $components[$component_name] = $component_data;
              $logger->info('Successfully parsed SDC component: @name with @props props', [
                '@name' => $component_name,
                '@props' => count($component_data['props'] ?? []),
              ]);
            }
            else {
              $logger->warning('Component data validation failed for: @name', ['@name' => $component_name]);
            }
          }
          else {
            $logger->debug('No .component.yml found or not readable in: @dir', ['@dir' => $component_name]);
          }
        }
        catch (\Exception $e) {
          $logger->error('Error processing component directory @name: @error', [
            '@name' => $item->getBasename(),
            '@error' => $e->getMessage(),
          ]);
        }
      }
    }
    catch (\Exception $e) {
      $logger->error('Error scanning SDC directory @dir: @error', [
        '@dir' => $directory,
        '@error' => $e->getMessage(),
      ]);
    }

    return $components;
  }

  /**
   * Parse individual SDC component with comprehensive validation.
   */
  protected function parseSdcComponent(string $component_dir, string $component_name, string $yml_file): ?array {
    $logger = $this->loggerFactory->get('component_field');

    try {
      // SAFETY: Validate input parameters.
      if (empty($component_dir) || empty($component_name) || empty($yml_file)) {
        $logger->warning('Invalid parameters for parseSdcComponent');
        return NULL;
      }

      if (!file_exists($yml_file) || !is_readable($yml_file)) {
        $logger->warning('YAML file does not exist or is not readable: @file', ['@file' => $yml_file]);
        return NULL;
      }

      // Parse YAML metadata with error handling.
      $yaml_content = file_get_contents($yml_file);
      if ($yaml_content === FALSE || empty($yaml_content)) {
        $logger->warning('Empty or unreadable YAML file for component: @name', ['@name' => $component_name]);
        return NULL;
      }

      try {
        $metadata = Yaml::parse($yaml_content);
      }
      catch (\Exception $e) {
        $logger->error('YAML parsing failed for component @name: @error', [
          '@name' => $component_name,
          '@error' => $e->getMessage(),
        ]);
        return NULL;
      }

      if (!is_array($metadata) || empty($metadata)) {
        $logger->warning('Invalid YAML metadata for component: @name', ['@name' => $component_name]);
        return NULL;
      }

      // Extract component info with proper validation and defaults.
      $component_data = [
        'name' => $component_name,
        'label' => $this->getStringValue($metadata, 'name', $this->humanizeComponentName($component_name)),
        'description' => $this->getStringValue($metadata, 'description', ''),
        'path' => $component_dir,
        'yml_file' => $yml_file,
        'relative_path' => str_replace(DRUPAL_ROOT . '/', '', $component_dir),
        'modified' => filemtime($yml_file) ?: time(),
        'metadata' => $metadata,
        'props' => [],
        'form_config' => [],
      ];

      // Parse props from YAML schema with validation.
      if (isset($metadata['props']['properties']) && is_array($metadata['props']['properties'])) {
        $component_data['props'] = $metadata['props']['properties'];

        // Generate form config with error handling.
        try {
          $form_config = $this->generateFormConfigFromYaml($metadata['props']['properties']);
          $component_data['form_config'] = is_array($form_config) ? $form_config : [];

          $logger->info('SDC component @name has @prop_count props', [
            '@name' => $component_name,
            '@prop_count' => count($component_data['props']),
          ]);
        }
        catch (\Exception $e) {
          $logger->error('Error generating form config for @name: @error', [
            '@name' => $component_name,
            '@error' => $e->getMessage(),
          ]);
          $component_data['form_config'] = [];
        }
      }
      else {
        $logger->debug('SDC component @name has no props defined', ['@name' => $component_name]);
      }

      // Check for additional files with validation.
      $this->addOptionalFiles($component_data, $component_dir, $component_name);

      return $component_data;

    }
    catch (\Exception $e) {
      $logger->error('Error parsing SDC component @name: @error', [
        '@name' => $component_name,
        '@error' => $e->getMessage(),
      ]);
      return NULL;
    }
  }

  /**
   * Safely get string value from array with default.
   */
  protected function getStringValue(array $array, string $key, string $default = ''): string {
    if (isset($array[$key]) && is_string($array[$key])) {
      return $array[$key];
    }
    return $default;
  }

  /**
   * Add optional files to component data with validation.
   */
  protected function addOptionalFiles(array &$component_data, string $component_dir, string $component_name): void {
    $files = [
      'template_file' => $component_name . '.twig',
      'css_file' => $component_name . '.css',
      'js_file' => $component_name . '.js',
    ];

    foreach ($files as $key => $filename) {
      $file_path = $component_dir . '/' . $filename;
      if (file_exists($file_path) && is_readable($file_path)) {
        $component_data[$key] = $file_path;
      }
    }
  }

  /**
   * Generate form configuration from YAML props schema with robust error handling.
   */
  protected function generateFormConfigFromYaml(array $props): array {
    if (!is_array($props) || empty($props)) {
      return [];
    }

    $config = [];

    foreach ($props as $prop_name => $prop_schema) {
      try {
        // SAFETY: Validate prop name and schema.
        if (!is_string($prop_name) || empty($prop_name) || !is_array($prop_schema)) {
          continue;
        }

        $form_element = $this->yamlPropToFormElement($prop_name, $prop_schema);
        if (is_array($form_element) && !empty($form_element)) {
          $config[$prop_name] = $form_element;
        }
      }
      catch (\Exception $e) {
        \Drupal::logger('component_field')->warning('Error processing prop @prop: @error', [
          '@prop' => $prop_name,
          '@error' => $e->getMessage(),
        ]);
      }
    }

    return $config;
  }

  /**
   * Build select options safely.
   */
  protected function buildSelectOptions(array $options): array {
    $safe_options = [];

    foreach ($options as $option) {
      if (is_scalar($option)) {
        $safe_options[(string) $option] = (string) $option;
      }
    }

    return $safe_options;
  }

  /**
   * Build entity reference element with validation.
   */
  protected function buildEntityReferenceElement(array $element, array $prop_schema): array {
    try {
      $target_type = $this->getStringValue($prop_schema, 'target_type', 'node');
      $target_bundles = isset($prop_schema['target_bundles']) && is_array($prop_schema['target_bundles']) ? $prop_schema['target_bundles'] : [];

      switch ($target_type) {
        case 'media':
          $element['#type'] = 'media_library';
          $element['#allowed_bundles'] = !empty($target_bundles) ? $target_bundles : ['image', 'video'];
          $element['#title'] .= ' (Media)';
          break;

        case 'node':
          $element['#type'] = 'entity_autocomplete';
          $element['#target_type'] = 'node';
          if (!empty($target_bundles)) {
            $element['#selection_settings']['target_bundles'] = array_combine($target_bundles, $target_bundles);
          }
          $element['#title'] .= ' (Content)';
          break;

        case 'taxonomy_term':
          $element['#type'] = 'entity_autocomplete';
          $element['#target_type'] = 'taxonomy_term';
          if (!empty($target_bundles)) {
            $element['#selection_settings']['target_bundles'] = array_combine($target_bundles, $target_bundles);
          }
          $element['#title'] .= ' (Terms)';
          break;

        default:
          // Fallback to autocomplete.
          $element['#type'] = 'entity_autocomplete';
          $element['#target_type'] = $target_type;
          break;
      }

      return $element;

    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error building entity reference element: @error', ['@error' => $e->getMessage()]);

      // Fallback to textfield.
      $element['#type'] = 'textfield';
      $element['#title'] .= ' (Entity ID)';
      return $element;
    }
  }

  /**
   * Build array element with advanced support and validation.
   */
  protected function buildArrayElement(array $element, array $prop_schema): array {
    try {
      $items_schema = isset($prop_schema['items']) && is_array($prop_schema['items']) ? $prop_schema['items'] : [];

      if (!empty($items_schema) && isset($items_schema['type']) && $items_schema['type'] === 'object') {
        // Complex object array - use a more advanced widget.
        $element['#type'] = 'details';
        $element['#title'] = $element['#title'] . ' (Advanced)';
        $element['#open'] = FALSE;

        $element['json_input'] = [
          '#type' => 'textarea',
          '#title' => t('JSON Configuration'),
          '#rows' => 6,
          '#description' => t('Enter as JSON array. Example:') . '<br><code>' .
          json_encode($this->generateExampleArray($items_schema), JSON_PRETTY_PRINT) . '</code>',
          '#element_validate' => [[$this, 'validateJsonArray']],
        ];

        return $element;
      }
      else {
        // Simple array.
        $element['#type'] = 'textarea';
        $element['#rows'] = 3;
        $element['#title'] .= ' (JSON Array)';
        $element['#description'] .= $element['#description'] ? '<br>' : '';
        $element['#description'] .= 'Enter as JSON array, e.g., ["item1", "item2"]';
        $element['#element_validate'] = [[$this, 'validateJsonArray']];

        return $element;
      }
    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error building array element: @error', ['@error' => $e->getMessage()]);

      // Fallback to simple textarea.
      $element['#type'] = 'textarea';
      $element['#rows'] = 3;
      $element['#title'] .= ' (JSON Array)';
      return $element;
    }
  }

  /**
   * Generate example array for documentation with validation.
   */
  protected function generateExampleArray(array $items_schema): array {
    $example = [];

    try {
      if (isset($items_schema['properties']) && is_array($items_schema['properties'])) {
        foreach ($items_schema['properties'] as $prop_name => $prop_def) {
          if (!is_array($prop_def) || !is_string($prop_name)) {
            continue;
          }

          $prop_type = $this->getStringValue($prop_def, 'type', 'string');

          switch ($prop_type) {
            case 'string':
              $example[$prop_name] = $this->getStringValue($prop_def, 'default', 'example');
              break;

            case 'boolean':
              $example[$prop_name] = isset($prop_def['default']) ? (bool) $prop_def['default'] : TRUE;
              break;

            case 'number':
            case 'integer':
              $example[$prop_name] = isset($prop_def['default']) && is_numeric($prop_def['default']) ? $prop_def['default'] : 1;
              break;
          }
        }
      }
    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->warning('Error generating example array: @error', ['@error' => $e->getMessage()]);
    }

    return [$example];
  }

  /**
   * Validate JSON array input.
   */
  public function validateJsonArray($element, $form_state) {
    $value = $element['#value'] ?? '';
    if (!empty($value)) {
      $decoded = json_decode($value, TRUE);
      if (json_last_error() !== JSON_ERROR_NONE || !is_array($decoded)) {
        $form_state->setError($element, t('Please enter a valid JSON array.'));
      }
    }
  }

  /**
   * Validate JSON object input.
   */
  public function validateJsonObject($element, $form_state) {
    $value = $element['#value'] ?? '';
    if (!empty($value)) {
      $decoded = json_decode($value, TRUE);
      if (json_last_error() !== JSON_ERROR_NONE || !is_array($decoded)) {
        $form_state->setError($element, t('Please enter a valid JSON object.'));
      }
    }
  }

  /**
   * Convert component name to human readable with validation.
   */
  protected function humanizeComponentName(string $name): string {
    if (empty($name) || !is_string($name)) {
      return 'Unknown Component';
    }

    try {
      // Convert kebab-case and underscore to Title Case.
      $result = str_replace(['-', '_'], ' ', $name);
      $result = preg_replace('/^component\s+/i', 'Component ', $result);
      return ucwords($result);
    }
    catch (\Exception $e) {
      return 'Unknown Component';
    }
  }

  /**
   * Convert prop name to human readable with validation.
   */
  protected function humanizePropName(string $name): string {
    if (empty($name) || !is_string($name)) {
      return 'Property';
    }

    try {
      // Convert camelCase to Title Case.
      $result = preg_replace('/([a-z])([A-Z])/', '$1 $2', $name);
      return ucfirst($result);
    }
    catch (\Exception $e) {
      return 'Property';
    }
  }

  /**
   * Clear component cache.
   */
  public function clearCache(): void {
    try {
      $this->cache->invalidate(self::CACHE_ID);
    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->error('Error clearing cache: @error', ['@error' => $e->getMessage()]);
    }
  }

  /**
   * Get single component info with validation.
   */
  public function getComponent(string $name): ?array {
    try {
      if (empty($name) || !is_string($name)) {
        return NULL;
      }

      $components = $this->discoverComponents();
      return isset($components[$name]) && is_array($components[$name]) ? $components[$name] : NULL;
    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->error('Error getting component @name: @error', [
        '@name' => $name,
        '@error' => $e->getMessage(),
      ]);
      return NULL;
    }
  }

  /**
   * Get component count with error handling.
   */
  public function getComponentCount(): int {
    try {
      $components = $this->discoverComponents();
      return is_array($components) ? count($components) : 0;
    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->error('Error getting component count: @error', ['@error' => $e->getMessage()]);
      return 0;
    }
  }

  /**
   * Check if component exists with validation.
   */
  public function componentExists(string $name): bool {
    try {
      if (empty($name) || !is_string($name)) {
        return FALSE;
      }

      $components = $this->discoverComponents();
      return isset($components[$name]) && is_array($components[$name]);
    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->error('Error checking component existence for @name: @error', [
        '@name' => $name,
        '@error' => $e->getMessage(),
      ]);
      return FALSE;
    }
  }

  /**
   * Get components by type or pattern with validation.
   */
  public function getComponentsByPattern(string $pattern): array {
    try {
      if (empty($pattern) || !is_string($pattern)) {
        return [];
      }

      $components = $this->discoverComponents();
      $filtered = [];

      foreach ($components as $name => $component) {
        if (is_string($name) && is_array($component) && fnmatch($pattern, $name)) {
          $filtered[$name] = $component;
        }
      }

      return $filtered;
    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->error('Error getting components by pattern @pattern: @error', [
        '@pattern' => $pattern,
        '@error' => $e->getMessage(),
      ]);
      return [];
    }
  }

  /**
   * Get component statistics with comprehensive error handling.
   */
  public function getComponentStats(): array {
    $stats = [
      'total' => 0,
      'with_props' => 0,
      'with_templates' => 0,
      'with_css' => 0,
      'with_js' => 0,
      'total_props' => 0,
    ];

    try {
      $components = $this->discoverComponents();

      if (!is_array($components)) {
        return $stats;
      }

      $stats['total'] = count($components);

      foreach ($components as $component) {
        if (!is_array($component)) {
          continue;
        }

        if (!empty($component['props']) && is_array($component['props'])) {
          $stats['with_props']++;
          $stats['total_props'] += count($component['props']);
        }
        if (!empty($component['template_file']) && is_string($component['template_file'])) {
          $stats['with_templates']++;
        }
        if (!empty($component['css_file']) && is_string($component['css_file'])) {
          $stats['with_css']++;
        }
        if (!empty($component['js_file']) && is_string($component['js_file'])) {
          $stats['with_js']++;
        }
      }
    }
    catch (\Exception $e) {
      \Drupal::logger('component_field')->error('Error getting component stats: @error', ['@error' => $e->getMessage()]);
    }

    return $stats;
  }

}
