<?php

namespace Drupal\component_field\Service;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;

/**
 * Service for parsing component props from source files.
 */
class PropsParser {

  use DependencySerializationTrait;

  /**
   * The logger factory.
   */
  protected LoggerChannelFactoryInterface $loggerFactory;

  /**
   * Constructor.
   */
  public function __construct(LoggerChannelFactoryInterface $logger_factory) {
    $this->loggerFactory = $logger_factory;
  }

  /**
   * Parse component file for props.
   */
  public function parseComponentFile(string $file_path): array {
    if (!file_exists($file_path)) {
      return [];
    }

    $content = file_get_contents($file_path);
    $props = [];

    // Try different parsing strategies.
    $props = array_merge($props, $this->parseTypeScriptInterface($content));
    $props = array_merge($props, $this->parsePropTypes($content));
    $props = array_merge($props, $this->parseJSDocComments($content));
    $props = array_merge($props, $this->parseDefaultProps($content));

    return $props;
  }

  /**
   * Parse TypeScript interface definitions.
   */
  protected function parseTypeScriptInterface(string $content): array {
    $props = [];

    // Look for interface definitions.
    $pattern = '/interface\s+\w*Props?\s*{([^}]+)}/';
    if (preg_match($pattern, $content, $matches)) {
      $interface_body = $matches[1];

      // Parse individual properties.
      $prop_pattern = '/(\w+)(\?)?:\s*([^;]+);?/';
      if (preg_match_all($prop_pattern, $interface_body, $prop_matches, PREG_SET_ORDER)) {
        foreach ($prop_matches as $match) {
          $prop_name = $match[1];
          $optional = !empty($match[2]);
          $type_def = trim($match[3]);

          $props[$prop_name] = $this->parseTypeDefinition($type_def);
          $props[$prop_name]['required'] = !$optional;
        }
      }
    }

    return $props;
  }

  /**
   * Parse PropTypes definitions.
   */
  protected function parsePropTypes(string $content): array {
    $props = [];

    // Look for PropTypes definitions.
    $pattern = '/\.propTypes\s*=\s*{([^}]+)}/';
    if (preg_match($pattern, $content, $matches)) {
      $proptypes_body = $matches[1];

      // Parse individual properties.
      $prop_pattern = '/(\w+):\s*PropTypes\.(\w+)(?:\.(\w+))?/';
      if (preg_match_all($prop_pattern, $proptypes_body, $prop_matches, PREG_SET_ORDER)) {
        foreach ($prop_matches as $match) {
          $prop_name = $match[1];
          $type = $match[2];
          $modifier = $match[3] ?? NULL;

          $props[$prop_name] = $this->propTypeToSchema($type, $modifier);
        }
      }
    }

    return $props;
  }

  /**
   * Parse JSDoc comments for prop documentation.
   */
  protected function parseJSDocComments(string $content): array {
    $props = [];

    // Look for @param annotations.
    $pattern = '/\*\s*@param\s*{([^}]+)}\s*(\w+)\s*(.*)$/m';
    if (preg_match_all($pattern, $content, $matches, PREG_SET_ORDER)) {
      foreach ($matches as $match) {
        $type = trim($match[1]);
        $prop_name = trim($match[2]);
        $description = trim($match[3]);

        if (!isset($props[$prop_name])) {
          $props[$prop_name] = [];
        }

        $props[$prop_name] = array_merge(
          $props[$prop_name],
          $this->parseTypeDefinition($type)
        );

        if ($description) {
          $props[$prop_name]['description'] = $description;
        }
      }
    }

    return $props;
  }

  /**
   * Parse default props.
   */
  protected function parseDefaultProps(string $content): array {
    $props = [];

    // Look for defaultProps.
    $pattern = '/\.defaultProps\s*=\s*{([^}]+)}/';
    if (preg_match($pattern, $content, $matches)) {
      $defaults_body = $matches[1];

      // Parse individual defaults.
      $prop_pattern = '/(\w+):\s*([^,}]+)/';
      if (preg_match_all($prop_pattern, $defaults_body, $prop_matches, PREG_SET_ORDER)) {
        foreach ($prop_matches as $match) {
          $prop_name = $match[1];
          $default_value = trim($match[2], ' \'"');

          if (!isset($props[$prop_name])) {
            $props[$prop_name] = [];
          }

          $props[$prop_name]['default'] = $this->parseDefaultValue($default_value);
        }
      }
    }

    return $props;
  }

  /**
   * Parse type definition string.
   */
  protected function parseTypeDefinition(string $type_def): array {
    $type_def = trim($type_def);

    // Handle union types.
    if (strpos($type_def, '|') !== FALSE) {
      $types = array_map('trim', explode('|', $type_def));
      $non_null_types = array_filter($types, function ($t) {
        return $t !== 'null' && $t !== 'undefined';
      });

      if (count($non_null_types) === 1) {
        return $this->parseTypeDefinition($non_null_types[0]);
      }
      else {
        return [
          'type' => 'string',
          'enum' => $non_null_types,
        ];
      }
    }

    // Handle specific types.
    switch (strtolower($type_def)) {
      case 'string':
        return ['type' => 'string'];

      case 'number':
        return ['type' => 'number'];

      case 'boolean':
        return ['type' => 'boolean'];

      case 'array':
        return ['type' => 'array'];

      default:
        // Check for enum-like strings.
        if (preg_match('/[\'"]([^\'"]+)[\'"]/', $type_def, $matches)) {
          return [
            'type' => 'string',
            'enum' => [$matches[1]],
          ];
        }
        return ['type' => 'string'];
    }
  }

  /**
   * Convert PropTypes to schema.
   */
  protected function propTypeToSchema(string $type, ?string $modifier): array {
    $schema = [];

    switch ($type) {
      case 'string':
        $schema['type'] = 'string';
        break;

      case 'number':
        $schema['type'] = 'number';
        break;

      case 'bool':
        $schema['type'] = 'boolean';
        break;

      case 'array':
        $schema['type'] = 'array';
        break;

      case 'oneOf':
        $schema['type'] = 'string';
        // Would need additional parsing for enum values.
        break;

      default:
        $schema['type'] = 'string';
    }

    $schema['required'] = $modifier === 'isRequired';

    return $schema;
  }

  /**
   * Parse default value.
   */
  protected function parseDefaultValue(string $value): mixed {
    // Remove quotes.
    $value = trim($value, '\'"');

    // Try to parse as JSON for complex values.
    $json_decoded = json_decode($value, TRUE);
    if (json_last_error() === JSON_ERROR_NONE) {
      return $json_decoded;
    }

    // Handle booleans.
    if ($value === 'true') {
      return TRUE;
    }
    if ($value === 'false') {
      return FALSE;
    }

    // Handle numbers.
    if (is_numeric($value)) {
      return strpos($value, '.') !== FALSE ? (float) $value : (int) $value;
    }

    return $value;
  }

}
